<?php
/*********************************************************************************************
 * e107 PLUGIN: Auto Gallery
 * VERSION:     2.xx
 * DESCRIPTION: A media/image gallery, where galleries are based on a directory structure. 
 *              Thumbnails are automatically generated through Imagemagick or GD.
 * WRITTEN BY:  Mr_Visible (www.cerebralsynergy.com)
 * DATE:        11/04/2007
 *
 * Released under the terms and conditions of the
 * GNU General Public License (http://gnu.org).
 ********************************************************************************************/
 
error_reporting(E_ALL);

if (!defined("AUTOGAL_CHARSET"))
{
	if (defined("CHARSET")){
		define("AUTOGAL_CHARSET", CHARSET);
	}else{
		define("AUTOGAL_CHARSET", "utf-8");
	}
}

class AutoGal_CMeta
{
	var $m_metaData;
	var $m_lastError;
	var $m_xmlParser;
	var $m_currentTag;
	var $m_currGroupData;
	var $m_DEBUG = 0;
	var $m_currGroup;
	var $m_allowedTags = array
	(
		'description', 'viewhits', 'emailhits', 'submitbyusername', 'submitbyuserid', 
		'submitdate', 'viewclass', 'uploadclass', 'adminclass', 'gcommentclass', 
		'mcommentclass', 'suggestedgallery', 'viewwidth', 'viewheight',
	);
	
	var $m_allowedGroupTags = array
	(
		'comments', 'ratings', 'arcade'
	);
	
	function GetLastError()
	{
		return $this->m_lastError;
	}
	
	function LoadFromGetMeta($getMetaInfo)
	{
		$this->m_metaData = array();
		$metaTags = array_merge($this->m_allowedTags, $this->m_allowedGroupTags);
		
		foreach ($metaTags as $tagName)
		{
			if ($this->m_DEBUG) print "LOAD FROM META: $tagName = ".$getMetaInfo[$tagName]."<br>";
			if ($tagName != '')
			{
				$this->m_metaData[$tagName] = $getMetaInfo[$tagName];
			}
		}
	}
	
	function LoadFile($fileName)
	{
		$this->m_metaData = array();
		$this->m_lastError = '';
		$this->m_xmlParser = NULL;
		$this->m_currentTag = '';
		$this->m_currComment = array();
		$this->m_currGroup = array();
				
		// CHECK IF XML PARSER EXISTS
		if(!function_exists('xml_parser_create'))
		{
			$this->m_lastError = AUTOGAL_LANG_METACLASS_L1;
			if ($this->m_DEBUG) print AUTOGAL_LANG_METACLASS_L2.$this->m_lastError."<br />\n";
			return false;
		}
		
		// READ FILE IN
		#print "FOPEN($fileName, r)<br />";
		$HANDLE = fopen($fileName, "r");
		
		if (!$HANDLE)
		{
			$this->m_lastError = str_replace('[FILE]', $fileName, AUTOGAL_LANG_METACLASS_L3);
			if ($this->m_DEBUG) print AUTOGAL_LANG_METACLASS_L2.$this->m_lastError."<br />\n";
			return false;
		}
		
		$xmlData = fread($HANDLE, filesize($fileName));
		
		if (!$xmlData)
		{
			$this->m_lastError = str_replace('[FILE]', $fileName, AUTOGAL_LANG_METACLASS_L4);
			if ($this->m_DEBUG) print AUTOGAL_LANG_METACLASS_L2.$this->m_lastError."<br />\n";
			return false;
		}
		
		fclose($HANDLE);
		
		// START XML PARSER
		$this->m_xmlParser = xml_parser_create('');
		xml_set_object($this->m_xmlParser, $this);
		xml_set_element_handler($this->m_xmlParser, 'StartElement', 'EndElement');
		xml_set_character_data_handler($this->m_xmlParser, 'CharacterData'); 
		xml_parse($this->m_xmlParser, $xmlData);
		
		xml_parser_free($this->m_xmlParser );
		
		// DECODE DATES
		foreach ($this->m_metaData as $field => $val)
		{
			if ((!is_array($val))&&(preg_match("/date$/", $field)))
			{
				$this->m_metaData[$field] = $this->DecodeDate($val);
			}
		}
			
		// SORT THE COMMENTS
		usort($this->m_metaData['comments'], "AutoGal_CompareComments");

		return true;
	}
	
	function StartElement($parser, $element, &$attrs)
	{
		$element = strtolower($element);
		$currGroup = $this->m_currGroup[count($this->m_currGroup) - 1];
		
		if ($this->m_DEBUG) print "START $element<br>";
		
		if ($element == 'autogalmeta')
		{
			$this->m_currGroup[] = 'autogalmeta';
		}
		elseif ($currGroup == 'autogalmeta')
		{
			if ($element == 'comments')
			{
				$this->m_currGroup[] = 'comments';
			}
			else if ($element == 'arcade')
			{
				$this->m_currGroup[] = 'arcade';
			}
			else if ($element == 'ratings')
			{
				$this->m_currGroup[] = 'ratings';
			}
		}
		elseif ($currGroup == 'comments')
		{
			if ($element == 'post')
			{
				$this->m_currGroup[] = 'post';
			}
		}
		elseif ($currGroup == 'arcade')
		{
			if ($element == 'topscores')
			{
				$this->m_currGroup[] = 'topscores';
			}
		}
		elseif ($currGroup == 'topscores')
		{
			if ($element == 'score')
			{
				$this->m_currGroup[] = 'score';
			}
		}
		elseif ($currGroup == 'ratings')
		{
			if ($element == 'rating')
			{
				$this->m_currGroup[] = 'rating';
			}
		}
		else
		{
			if ($this->m_DEBUG) print "START BAD: $element (Group: $currGroup)<br />";
		}
						
		$this->m_currentTag = $element;
	}
		
	function EndElement($parser, $element)
	{
		if (count($this->m_currGroup) <= 0) return;
		
		$element = strtolower($element);
		$currGroup = $this->m_currGroup[count($this->m_currGroup) - 1];
		
		if ($this->m_DEBUG) print "END $element (group $currGroup)<br>";
			
		if ($element == $currGroup)
		{
			array_pop($this->m_currGroup);
			
			if ($element == 'post')
			{
				$this->m_currGroupData['date'] = $this->DecodeDate($this->m_currGroupData['date']);
				if ($this->m_currGroupData['date'] >= 0)
				{
					$this->m_metaData['comments'][] = $this->m_currGroupData;
				}
			}
			elseif ($element == 'rating')
			{
				$this->m_currGroupData['date'] = $this->DecodeDate($this->m_currGroupData['date']);
				if ($this->m_currGroupData['date'] > 0)
				{
					$this->m_metaData['ratings'][] = $this->m_currGroupData;
				}
			}
			elseif ($element == 'score')
			{
				$this->m_currGroupData['date'] = $this->DecodeDate($this->m_currGroupData['date']);
				if ($this->m_currGroupData['date'] > 0)
				{
					$this->m_metaData['arcade']['topscores'][] = $this->m_currGroupData;
				}
			}
			$this->m_currGroupData = array();
		}
			
		$this->m_currentTag = "";
	}

	function CharacterData ($parser, $data)
	{
		if (count($this->m_currGroup) <= 0) return;
		
		$this->m_currentTag = strtolower($this->m_currentTag);
		$currGroup = $this->m_currGroup[count($this->m_currGroup) - 1];
		
		if ($this->m_DEBUG) print "DATA ".$this->m_currentTag."/".$currGroup."($data)<br />";
		
		if ($currGroup == 'post')
		{
			// COMMENT POST
			if ($this->m_currentTag == 'text')
			{
				$this->m_currGroupData['text'] .= $data;
			}
			elseif ($this->m_currentTag == 'date')
			{
				$this->m_currGroupData['date'] .= $data;
			}
			elseif ($this->m_currentTag == 'authorid')
			{
				$this->m_currGroupData['authorid'] .= $data;
			}
			elseif ($this->m_currentTag == 'authorusername')
			{
				$this->m_currGroupData['authorusername'] .= $data;
			}
		}
		else if ($currGroup == 'rating')
		{
			// RATING
			if ($this->m_currentTag == 'userid')
			{
				$this->m_currGroupData['userid'] .= $data;
			}
			elseif ($this->m_currentTag == 'username')
			{
				$this->m_currGroupData['username'] .= $data;
			}
			elseif ($this->m_currentTag == 'date')
			{
				$this->m_currGroupData['date'] .= $data;
			}
			elseif ($this->m_currentTag == 'mark')
			{
				$this->m_currGroupData['mark'] .= $data;
			}
		}
		else if ($currGroup == 'score')
		{
			// ARCADE SCORE
			if ($this->m_currentTag == 'userid')
			{
				$this->m_currGroupData['userid'] .= $data;
			}
			elseif ($this->m_currentTag == 'username')
			{
				$this->m_currGroupData['username'] .= $data;
			}
			elseif ($this->m_currentTag == 'date')
			{
				$this->m_currGroupData['date'] .= $data;
			}
			elseif ($this->m_currentTag == 'points')
			{
				$this->m_currGroupData['points'] .= $data;
			}
		}
		elseif (in_array($this->m_currentTag, $this->m_allowedTags))
		{
			$this->m_metaData[$this->m_currentTag] .= $data;
		}
	}
	
	function SaveFile($fileName, $perms=777)
	{
		// BUILD XML META DATA
		$xmlData .=
		"<?xml version=\"1.0\" encoding=\"".AUTOGAL_CHARSET."\"?>\n\n".
		"<!-- Auto Gallery (www.cerebralsynergy.com) XML Meta File -->\n\n".
		"<autogalmeta>\n";
		
		// WRITE OUT NORMAL TAGS
		if ($this->m_DEBUG)
		{	
			AutoGal_Dump($this->m_metaData);
		}
				
		foreach ($this->m_metaData as $tagName => $tagValue)
		{
			if (($tagValue <> '')&&(in_array($tagName, $this->m_allowedTags))&&($tagName != 'comments'))
			{
				if ($this->m_DEBUG) print "SAVE: $tagName => $tagValue<br>";
				
				if (is_array($tagValue))
				{
					$xmlData .= "<$tagName>\n";
					foreach ($tagValue as $subTagName => $subTagValue)
					{
						$xmlData .= "\t<$subTagName>".htmlspecialchars($subTagValue)."</$subTagName>\n";
					}
					$xmlData .= "</$tagName>\n";
				}
				else
				{
					if (preg_match("/date$/", $tagName))
					{
						$xmlData .= "<$tagName>".htmlspecialchars(strftime("%H:%M:%S %d-%b-%Y", $tagValue))."</$tagName>\n";
					}
					else
					{
						$xmlData .= "<$tagName>".htmlspecialchars($tagValue)."</$tagName>\n";
					}
				}
			}
		}
		
		// WRITE OUT COMMENTS
		if (count($this->m_metaData['comments']) > 0)
		{
			$xmlData .= "<comments>\n";
			
			foreach ($this->m_metaData['comments'] as $commentI => $comment)
			{
				$xmlData .= "\t<post>\n";
				$xmlData .= "\t\t<authorid>".htmlspecialchars($comment['authorid'])."</authorid>\n";
				$xmlData .= "\t\t<authorusername>".htmlspecialchars($comment['authorusername'])."</authorusername>\n";
				$xmlData .= "\t\t<date>".htmlspecialchars(strftime("%H:%M:%S %d-%b-%Y", $comment['date']))."</date>\n";
				$xmlData .= "\t\t<text>".htmlspecialchars($comment['text'])."</text>\n";
				$xmlData .= "\t</post>\n";
			}
			
			$xmlData .= "</comments>\n";
		}
		
		if (count($this->m_metaData['ratings']) > 0)
		{
			$xmlData .= "<ratings>\n";
			
			foreach ($this->m_metaData['ratings'] as $ratingI => $rating)
			{
				$xmlData .= "\t<rating>\n";
				$xmlData .= "\t\t<userid>".htmlspecialchars($rating['userid'])."</userid>\n";
				$xmlData .= "\t\t<username>".htmlspecialchars($rating['username'])."</username>\n";
				$xmlData .= "\t\t<date>".htmlspecialchars(strftime("%H:%M:%S %d-%b-%Y", $rating['date']))."</date>\n";
				$xmlData .= "\t\t<mark>".htmlspecialchars($rating['mark'])."</mark>\n";
				$xmlData .= "\t</rating>\n";
			}
			
			$xmlData .= "</ratings>\n";
		}
		
		// WRITE OUT ARCADE DATA
		if (count($this->m_metaData['arcade']) > 0)
		{
			$xmlData .= "<arcade>\n";
			
			// TOP SCORES
			if (count($this->m_metaData['arcade']['topscores']) > 0)
			{
				$xmlData .= "\t<topscores>\n";
			
				foreach ($this->m_metaData['arcade']['topscores'] as $scoreI => $score)
				{
					$xmlData .= "\t\t<score>\n";
					$xmlData .= "\t\t\t<userid>".htmlspecialchars($score['userid'])."</userid>\n";
					$xmlData .= "\t\t\t<username>".htmlspecialchars($score['username'])."</username>\n";
					$xmlData .= "\t\t\t<date>".htmlspecialchars(strftime("%H:%M:%S %d-%b-%Y", $score['date']))."</date>\n";
					$xmlData .= "\t\t\t<points>".htmlspecialchars($score['points'])."</points>\n";
					$xmlData .= "\t\t</score>\n";
				}
			
				$xmlData .= "\t</topscores>\n";
			}
			
			$xmlData .= "</arcade>\n";
		}
		
		$xmlData .= "</autogalmeta>\n";
			
		// TOUCH AND CHMOD XML FILE IF NEW
		if (!file_exists($fileName))
		{
			touch($fileName);
			chmod($fileName, octdec($perms));
		}
		
		// WRITE XML TO FILE
		#print "FOPEN($fileName, w+)<br />";
		$HANDLE = fopen($fileName, "w+");
				
		if (!$HANDLE)
		{
			$this->m_lastError = str_replace('[FILE]', $fileName, AUTOGAL_LANG_METACLASS_L5);
			if ($this->m_DEBUG) print "ERROR: ".$this->m_lastError."<br />\n";
			flock($HANDLE, LOCK_UN);
			return false;
		}
		
		flock($HANDLE, LOCK_EX);
		if (fwrite($HANDLE, $xmlData) === false)
		{
			$this->m_lastError = str_replace('[FILE]', $fileName, AUTOGAL_LANG_METACLASS_L6);
			if ($this->m_DEBUG) print "ERROR: ".$this->m_lastError."<br />\n";
			flock($HANDLE, LOCK_UN);
			return false;
		}
		
		flock($HANDLE, LOCK_UN);
		fclose($HANDLE);
		
		if (!file_exists($fileName))
		{
			$this->m_lastError = str_replace('[FILE]', $fileName, AUTOGAL_LANG_METACLASS_L8);
			return false;
		}
		
		/*if (filesize($fileName) <= 0)
		{
			$this->m_lastError = str_replace('[FILE]', $fileName, AUTOGAL_LANG_METACLASS_L9);
			return false;
		}*/
		
		return true;
	}
	
	/*function SetMetaData($metaData)
	{
		$this->m_metaData = $metaData;
	}*/
	
	function GetMetaData()
	{
		return $this->m_metaData;
	}
	
	function SetMetaData($getMetaInfo)
	{
		$this->m_metaData = array();
		
		foreach ($this->m_allowedTags as $tagName)
		{
			if ($this->m_DEBUG) print "LOAD FROM META (S): $tagName = ".$getMetaInfo[$tagName]."<br />";
			if ($tagName != '')
			{
				$this->m_metaData[$tagName] = $getMetaInfo[$tagName];
			}
		}
		foreach ($this->m_allowedGroupTags as $tagName)
		{
			if ($this->m_DEBUG) print "LOAD FROM META (G): $tagName = ".$getMetaInfo[$tagName]."<br />";
			if ($tagName != '')
			{
				$this->m_metaData[$tagName] = $getMetaInfo[$tagName];
			}
		}
	}
	
	// WORK OUT EPOCH, CAN'T USE strptime 'COS IT'S PHP5 (NOONE SEEMS TO HAVE IT)
	function DecodeDate($dateStr)
	{
		if (preg_match("/^\s*(\d+)\:(\d+)\:(\d+)\s+(\d+)\-(\w+)\-(\d+)\s*$/", $dateStr, $dBits))
		{
			switch (strtolower($dBits[5]))
			{
				case 'jan': $monD = 1; break;
				case 'feb': $monD = 2; break;
				case 'mar': $monD = 3; break;
				case 'apr': $monD = 4; break;
				case 'may': $monD = 5; break;
				case 'jun': $monD = 6; break;
				case 'jul': $monD = 7; break;
				case 'aug': $monD = 8; break;
				case 'sep': $monD = 9; break;
				case 'oct': $monD = 10; break;
				case 'nov': $monD = 11; break;
				case 'dec': $monD = 12; break;
			}
			
			return mktime($dBits[1], $dBits[2], $dBits[3], $monD, $dBits[4], $dBits[6]);
		}
		else
		{
			return -1;
		}
	}
}

if (!function_exists('AutoGal_CompareComments'))
{
	function AutoGal_CompareComments($a, $b)
	{
		if ($a['date'] > $b['date']) return 1;
		if ($a['date'] < $b['date']) return -1;
		return 0;
	}
}
?>