<?php
/*********************************************************************************************
 * e107 PLUGIN: Auto Gallery
 * VERSION:     2.xx
 * DESCRIPTION: A media/image gallery, where galleries are based on a directory structure. 
 *              Thumbnails are automatically generated through Imagemagick or GD.
 * WRITTEN BY:  Mr_Visible (www.cerebralsynergy.com)
 * DATE:        11/04/2007
 *
 * Released under the terms and conditions of the
 * GNU General Public License (http://gnu.org).
 ********************************************************************************************/

error_reporting(E_ALL);

#print __FILE__;
#exit;
require_once(dirname(__FILE__)."/language.php");
require_once(dirname(__FILE__)."/../../class2.php");
require_once(dirname(__FILE__)."/def_core.php");

$g_eleInfoCache = array();

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_NumUploads()
{
    $uploadCount = 0;
    
    $dh = opendir(AUTOGAL_UPLOADDIRABS);
    while ($file = readdir($dh))
    {
		if (preg_match("/^".preg_quote(AUTOGAL_THUMBPREFIX)."/", $file)) continue;
        if (AutoGal_IsSupportedFormat($file))
        { 
            $uploadCount ++;
        }
    }
    closedir($dh);
    
    return ($uploadCount);
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetReviewLink()
{
	if (check_class(AUTOGAL_ADMINREVIEWUC)) 
		return "[<a href=\"".AUTOGAL_REVIEWUPLOADS."\">".AUTOGAL_LANG_L21." (".AutoGal_NumUploads().")</a>]";
	else
		return "";
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_AddAdminLog($type, $message)
{
	if (USER){
		$username = USERNAME;
	}else{
		$username = AUTOGAL_LANG_L41;
	}
	
	$timeStr = strftime("%b %d %Y, %I:%M%p");
	$ADMINLOG = fopen(AUTOGAL_ADMINLOG, 'a+'); 
	
	if ($ADMINLOG)
	{
		flock($ADMINLOG, LOCK_EX);
		
		if (!fwrite($ADMINLOG, "$timeStr> $username [$type] $message\n")) 
		{
			print str_replace('[LOG]', AUTOGAL_ADMINLOG, AUTOGAL_LANG_L42)."<br />\n";
		}
		flock($ADMINLOG, LOCK_UN);
		fclose($ADMINLOG);
	}
	else
	{
		print str_replace('[LOG]', AUTOGAL_ADMINLOG, AUTOGAL_LANG_L43)."<br />\n";
	}
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetGalUserClass($gallery, $type)
{
    $eleInfo = AutoGal_GetElementInfo($gallery, true);
	$type .= 'class';
	$galleryClass = $eleInfo[$type];
	return ($galleryClass == '' ? 0 : $galleryClass);
}

/*****************************************************************************************************
 * RETURNS THE TITLE FOR IMAGE/GALLERY
 ****************************************************************************************************/
function AutoGal_GetTitle($location, $isDir, $part='full')
{
	$location = preg_replace("/\/$/", '', $location);
    $location = preg_replace("/^\//", '', $location);
	
	if ($location == '') $location = AUTOGAL_ROOTNAME;
    
    if (!$isDir)
    {
        $location = preg_replace("/\.\w+$/", '', $location);
    }
	    
    $title = $location;
    if (preg_match("/\/([^\/]+)$/", $location, $bits))
    {
        $title = $bits[1];
    }

    $title = str_replace('_', ' ', $title);
    
	// IDEA SUPPLIED KINDLY BY RIJK
    $title = preg_replace("/^\d+\./", "", $title);
	$fullTitle = $title;

	// SUBTITLE
	if (preg_match("/^(.+)\[(.*)\]$/", $title, $bits))
	{
		$title = $bits[1];
		$subTitle = $bits[2];
		$subTitle = htmlspecialchars($subTitle);
	}
	else
	{
		$subTitle = '';
	}
    
	// UPPERCASE FIRST LETTERS
	if (AUTOGAL_UCASETITLES)
	{
		$smallWordsStr = strtolower(str_replace(' ', '', AUTOGAL_SMALLWORDS));
		$smallWords = explode(',', $smallWordsStr);
	
		$title = ucwords($title);
		$fullTitle = ucwords($fullTitle);
		$subTitle = ucwords($subTitle);
		
		foreach ($smallWords as $word)
		{
			$title = str_replace(' '.ucfirst($word).' ', " $word ", $title);
			$title = preg_replace("/ ".ucfirst($word)."$/", " $word", $title);
			$fullTitle = str_replace(' '.ucfirst($word).' ', " $word ", $fullTitle);
			$fullTitle = preg_replace("/ ".ucfirst($word)."$/", " $word", $fullTitle);
			$subTitle = str_replace(' '.ucfirst($word).' ', " $word ", $subTitle);
			$subTitle = preg_replace("/ ".ucfirst($word)."$/", " $word", $subTitle);
		}
	}

	// FOR &'s ETC.
	$title = htmlspecialchars($title);
	$fullTitle = htmlspecialchars($fullTitle);
		
	// THIS IS TO KEEP BACKWARDS COMPATABILITY
	if ($part == 'title')
		return $title;
	elseif ($part == 'subtitle')
		return $subTitle;
	elseif ($part == 'full')
		return $fullTitle;
	elseif ($part == 'all')
		return (array($title, $subTitle, $fullTitle));
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetID($fileName)
{
	$file = basename($fileName);
			
	$id = -1;
    if (preg_match("/^(\d+)\..*/", $file, $IDBits))
    {
        $id = $IDBits[1];
    }

	return $id;
}

/*****************************************************************************************************
 * DETERMINES IF A FILE IS AN IMAGE BASED ONE EXTENSION
 ****************************************************************************************************/
function AutoGal_GallerySelect($selected = '', $excludeGallery = '', $excludeRecurse=0, $inforceAdmin=0) 
{
    $select = "<option value=''>".AutoGal_GetTitle(AUTOGAL_ROOTNAME, 1, 'title')."</option>\n";
    $gallerys = AutoGal_GalleryList();
    
    if ($excludeGallery == AutoGal_GetTitle(AUTOGAL_ROOTNAME, 1, 'title')) 
	{
		$select = '';
	}
	elseif ($excludeGallery)
	{ 
		$exGall = array_search($excludeGallery, $gallerys);
		if ($exGall >= 0)
		{
			unset ($gallerys[$exGall]);
			reset ($gallerys);
			
			if ($excludeRecurse)
			{
				$scanGalleries = $gallerys;
				$gallerys = array();
				foreach ($scanGalleries as $galIndex => $gallery)
				{
					$excludeGalleryReg = str_replace('/', "\\/", $excludeGallery);
					if (!preg_match("/^$excludeGalleryReg\//", $gallery))
					{
						$gallerys[] = $gallery;
					}
				}
			}
		}
	}   

	if ($inforceAdmin)
	{
		$scanGalleries = $gallerys;
		$gallerys = array();
		
		foreach ($scanGalleries as $galIndex => $gallery)
		{
			$galInfo = AutoGal_GetElementInfo($gallery, true);
			if (check_class($galInfo['adminclass']))
			{
				$gallerys[] = $gallery;
			}
		}
	}
   
    foreach ($gallerys as $gallery)
    {
        $galleryTitle = AutoGal_GetElementPathTitle($gallery, 0, " -> ");
        $select .= "<option value=\"".rawurlencode($gallery)."\"".($selected && $selected == $gallery ? ' selected="selected"' : '').">$galleryTitle</option>\n";
    }
    
    return $select;
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetDefaultImage($gallery, &$absPath)
{
    $imageExts = explode('|', AUTOGAL_IMAGEEXTS);
    
    $defImageName = AUTOGAL_DEFAULTIMAGE;
	$defImagePath = '';
    
    foreach ($imageExts as $ext)
    {
        if (is_readable(AutoGal_GetAbsGalPath("$gallery/$defImageName.$ext")))
        {
			$defImagePath = "$gallery/$defImageName.$ext";
			break;
        }
    }
	
	if (!$defImagePath)
	{
		if (AUTOGAL_RANDOMDEFAULTIMG)
		{
			$maxItems = rand(1, 50);
			$thumbList = AutoGal_ListDirectory(AutoGal_GetAbsGalPath($gallery), "^".AUTOGAL_THUMBPREFIX, true, false, false, false, $maxItems);
						
			if (!$thumbList) return '';
			
			$defImagePath = "$gallery/".basename($thumbList[rand(0, count($thumbList) - 1)]);
			return AUTOGAL_GALLERYDIR.'/'.$defImagePath;
		}
		else
		{
			return '';
		}
	}
	else
	{
		if (AUTOGAL_AUTOTHUMB)
		{
			$absPath = AutoGal_GetAbsGalPath($defImagePath);
			
			$image_stats = getimagesize($absPath);
			$imageWidth = $image_stats[0];
			$imageHeight = $image_stats[1];
			if (($imageWidth > AUTOGAL_DEFAULTIMGWIDTH)||($imageHeight > AUTOGAL_DEFAULTIMGHEIGHT))
			{
				return AUTOGAL_RESIZE."?img=".rawurlencode($defImagePath);
			}
			else
			{
				return AUTOGAL_GALLERYDIR.'/'.$defImagePath;
			}
		}
	}
	
    return $defImagePath;
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetImageGallery($image)
{
    $image = str_replace("\\", '/', $image);
    $image = str_replace('//', '/', $image);
    
    if (strpos($image, '/') === false)
    {
        return '';
    }
    
    $gallery = dirname($image);
    $gallery = ($gallery == '.' ? '' : $gallery);
   
    return $gallery;
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetGalleryElements($gallery, $recurse=false, $sortOrder='name')
{
	$gallery = preg_replace("/\/$/", "", $gallery);
	
	require_once(AUTOGAL_GALLERYLISTCLASS);
	
	$opts = array
	(
		'sortorder' => $sortOrder,
		'recurse' => $recurse,
		'cachmethod' => AUTOGAL_DIRCACHEMETHOD,
		'cachefilename' => AUTOGAL_DIRCACHEFILE,
	);
	
	$galList = new AutoGal_CGalleryList($gallery, $opts);
	$galList->ListGallery();
	
	if ($galList->IsError)
	{
		print "ERROR: ".$galList->LastError."<br />";
	}
	
	return $galList->Elements();
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetImageIndex($imageEle, $galEles)
{
    foreach ($galEles['files'] as $imageIndex => $image)
    {
        if ($image['element'] == $imageEle) return $imageIndex;
    }
    
    return -1;
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GalleryList($startGal = "", $incStart = false /* for backwards compatability (pre 2.0)*/ ) 
{
    $select = '';
	
    $galDir = AutoGal_GetAbsGalPath();
    $stack[] = AutoGal_GetAbsGalPath($startGal);
    $gallerys = array();
	if ($incStart) $gallerys[] = $startGal;
    
    while ($stack)
    {
        $current_dir = array_pop($stack);
        if ($dh = opendir($current_dir))
        {
            while ($file = readdir($dh))
            {
                if ($file <> '.' && $file <> '..')
                {
                    $current_file = "{$current_dir}/{$file}";
                    if (is_dir($current_file))
                    {
                        $stack[] = $current_file;
                        $galleryPath = str_replace($galDir, '', $current_file);
                        $galleryPath = preg_replace("/^\//", '', $galleryPath);
                        $gallerys[] = $galleryPath;
                    }
                }
            }
        }
    }
		
	usort($gallerys, "AutoGal_CmpGalleries");
	
    return $gallerys;
}

function AutoGal_CmpGalleries($a, $b)
{
	$a = strtolower($a);
	$b = strtolower($b);
	
	if ($a == $b) return 0;
	return ($a < $b) ? -1 : 1;
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetImageDimensions($image, $isFullPath=false)
{
    if (!$isFullPath) $image = AutoGal_GetAbsGalPath($image);
	$image_stats = getimagesize($image);
    $imageSize['w'] = $image_stats[0];
	$imageSize['h'] = $image_stats[1];
    
    return $imageSize; 
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetElementPathTitle($element, $incID = false, $seperator = '/')
{
	// TO BE SAFE
	$element = str_replace("\\", '/', $element);
    $element = preg_replace("/^\//", "", $element);
    $element = preg_replace("/\/$/", "", $element);
		
	$pathTitle = '';
	$absPath = AutoGal_GetAbsGalPath($element);
	
	if (!$element) return AutoGal_GetTitle(AUTOGAL_ROOTNAME, 1, 'title');
	if (strstr($element, '/'))
	{
		$pathBits = explode('/', $element);
		for ($pathBitIndex = 0; $pathBitIndex < count($pathBits) - 1; $pathBitIndex ++)
		{
			$bathBitID = AutoGal_GetID($pathBits[$pathBitIndex]);
			$pathTitle .= AutoGal_GetTitle($pathBits[$pathBitIndex], 1, 'title').($incID && $bathBitID >= 0 ? " (ID: $bathBitID)" : "").$seperator;
		}
	}
	
	$bathBitID = AutoGal_GetID($element);
	$pathTitle .= AutoGal_GetTitle($element, is_dir($absPath), 'title').($incID && $bathBitID >= 0 ? " (ID: $bathBitID)" : "");
	
	return $pathTitle;
}

/*****************************************************************************************************
 * ------ TEH UBER FUNCTION -------
 * GETS ALL THE META DATA ON A GALLERY OR IMAGE. 
 ****************************************************************************************************/
function AutoGal_GetElementInfo($element, $getMetaFile=false)
{
	global $g_startImage;
	global $g_eleInfoCache;
	
	# HACKY AND AWFUL, I NEED A CLASS FOR THIS, SORRY :(
	if ($g_eleInfoCache[$element])
	{
		$info = $g_eleInfoCache[$element];
		if ((!$g_eleInfoCache[$element]['xmlread'])&&($getMetaFile))
		{
			$metaInfo = AutoGal_GetElementXMLInfo($info['abspath']);
			foreach ($metaInfo as $metaField => $metaValue)
			{
				$info[$metaField] = $metaValue;
			}
		}
		return $info;
	}
	
	$info['isnonimg'] = 0;
    $info['element'] = $element;
	$info['element'] = str_replace("\\", '/', $info['element']);
    $info['element'] = preg_replace("/^\//", "", $info['element']);
    $info['element'] = preg_replace("/\/$/", "", $info['element']);
	
	$info['abspath'] = AutoGal_GetAbsGalPath($info['element']);
	
	$info['isroot'] = false;
	if ($info['element'] == '') $info['isroot'] = true;
	if (!file_exists($info['abspath'])) return (array());
	
	$eleInfo = pathinfo($info['abspath']);
    $info['dirname'] = $eleInfo['dirname'];
    $info['basename'] = $eleInfo['basename'];
	
	if ($info['isroot'])
	{
		$info['url'] = AUTOGAL_GALLERYDIR;
		$info['link'] = AUTOGAL_AUTOGALLERY;
		$info['linkargs'] = '';
		$info['backlink'] = $info['link'].($g_startImage ? "?start=$g_startImage" : '');
	}
	else
	{
		
		$info['url'] = AutoGal_GetImageURL($element);
		$info['linkargs'] = "?show=".rawurlencode($info['element']);
		$info['link'] = AUTOGAL_AUTOGALLERY.$info['linkargs'];
		$info['backlink'] = $info['link'].($g_startImage ? "&start=$g_startImage" : '');
	}
	
	#print $info['url'];
	#exit;
    
    $info['num'] = AutoGal_GetID($info['element']);
    $info['id'] = $info['num'];
	
	$info['pathtitle'] = AutoGal_GetElementPathTitle($info['element']);
	
	// GET XML META FILE
	if ($getMetaFile)
	{
		$metaInfo = AutoGal_GetElementXMLInfo($info['abspath']);
		foreach ($metaInfo as $metaField => $metaValue)
		{
			$info[$metaField] = $metaValue;
		}
	}

	if (!is_numeric($info['viewclass'])) $info['viewclass'] = AUTOGAL_DEFAULTVIEWUC;
	if (!is_numeric($info['uploadclass'])) $info['uploadclass'] = AUTOGAL_DEFAULTUPLOADUC;
	if (!is_numeric($info['adminclass'])) $info['adminclass'] = AUTOGAL_DEFAULTADMINUC;
	if (!is_numeric($info['mcommentclass'])) $info['mcommentclass'] = AUTOGAL_DEFAULTMCOMMENTUC;
	if (!is_numeric($info['gcommentclass'])) $info['gcommentclass'] = AUTOGAL_DEFAULTGCOMMENTUC;
	
	$info['gallery'] = AutoGal_GetImageGallery($info['element']);
		
	if (is_dir($info['abspath']))
    {
        $info['type'] = 'Gallery'; 
		$info['filetypetitle'] = AUTOGAL_LANG_FTYPE_GALL; 
		$titleBits = AutoGal_GetTitle($info['element'], 1, 'all');
        $info['title'] = $titleBits[0];
		$info['subtitle'] = $titleBits[1];
		$info['fulltitle'] = $titleBits[2];
        $info['defimg'] = ($info['isroot'] ? '' : AutoGal_GetDefaultImage($info['element']));
		$info['isgallery'] = true;
		$info['isfile'] = false;
		
        $info['defimghtml'] = ($info['defimg'] ? "<img title=\"".$info['title']."\" style='border:none' src=\"".$info['defimg']."\" alt=\"".$info['title']."\" /><br />" : '');
		$info['ahref'] = "<a href=\"".$info['backlink']."\" ";
    }                                             
    else
    {
        $info['type'] = 'Image';
		$info['filetypetitle'] = AUTOGAL_LANG_FTYPE_IMAGE; 
        $titleBits = AutoGal_GetTitle($info['element'], 0, 'all');
        $info['title'] = $titleBits[0];
		$info['subtitle'] = $titleBits[1];
		$info['fulltitle'] = $titleBits[2];
        $info['isimg'] = AutoGal_IsSupportedFormat($info['element']);
        
        $info['extension'] = $eleInfo['extension'];
        $info['appext'] = ($eleInfo['extension'] ? '.'.$eleInfo['extension'] : '');
		$info['isgalroot'] = ($info['gallery'] == '' ? true : false);
		$info['isgallery'] = false;
		$info['isfile'] = true;
		
		// OPEN IN NEW WINDOW STUFF
		if (AUTOGAL_SHOWINNEWWINDOW)
		{
			$windowLink = $info['link']."&newwindow=1";
			$windowLink = str_replace("'", "\\'", $windowLink);
			
			$statusTitle = str_replace("[TITLE]", $info['title'], AUTOGAL_LANG_L36);
			$statusTitle = str_replace("'", "\\'", $statusTitle);
			
			$openWindowJS = AutoGal_NewWindowJS('show', $windowLink, 'viewimage');
					
			$info['ahref'] = "<a href=\"javascript: void(0);\" onmouseover=\"window.status='$statusTitle'; return true;\" onmouseout=\"window.status=''; return true;\" onClick=\"javascript:".$openWindowJS."\" ";
		}
		else
		{
			$info['ahref'] = "<a href=\"".$info['backlink']."\" ";
		}
		
		// FLASH/MOVIES
		$ext = strtolower($info['extension']);
		if (preg_match("/^(".AUTOGAL_FLASHEXTS.")$/", $ext))
		{
			$info['isflash'] = true;
			$info['filetypetitle'] = AUTOGAL_LANG_FTYPE_FLASH;
		}
		elseif (preg_match("/^(".AUTOGAL_FLVEXTS.")$/", $ext))
		{
			$info['isflv'] = true;
			$info['filetypetitle'] = AUTOGAL_LANG_FTYPE_FLV;
		}
		elseif (preg_match("/^(".AUTOGAL_QUICKTIMEEXTS.")$/", $ext))
		{
			$info['isquicktime'] = true;
			$info['ismovie'] = true;
			$info['filetypetitle'] = AUTOGAL_LANG_FTYPE_QTIME;
		}
		elseif (preg_match("/^(".AUTOGAL_WINMEDIAEXTS.")$/", $ext))
		{
			$info['iswinmedia'] = true;
			$info['ismovie'] = true;
			$info['filetypetitle'] = AUTOGAL_LANG_FTYPE_WMA;
		}
		elseif (preg_match("/^(".AUTOGAL_WINMEDIAEXTS_A.")$/", $ext))
		{
			$info['iswinmedia'] = true;
			$info['isaudio'] = true;
			$info['filetypetitle'] = AUTOGAL_LANG_FTYPE_AUDIO;
		}
		elseif (preg_match("/^(".AUTOGAL_REALMEDIAEXTS.")$/", $ext))
		{
			$info['isrealmedia'] = true;
			$info['ismovie'] = true;
			$info['filetypetitle'] = AUTOGAL_LANG_FTYPE_RM;
		}
		elseif (preg_match("/^(".AUTOGAL_IMAGEEXTS.")$/", $ext))
		{
			$info['isimage'] = true;
			$info['filetypetitle'] = AUTOGAL_LANG_FTYPE_IMAGE;
		}
					
		if (($info['isflash'])||($info['isquicktime'])||($info['iswinmedia'])||($info['isrealmedia'])||($info['isflv']))
		{
			// THUMBNAILS FOR FLASH/MOVIES
			$info['isnonimg'] = 1;
			
			$possibleExts = explode('|', AUTOGAL_IMAGEEXTS);
			$thExt = '';
			foreach ($possibleExts as $testExt)
			{
				if ($testExt == $ext) continue;
				
				$testFile = $eleInfo['dirname'].'/'.AUTOGAL_THUMBPREFIX.$eleInfo['basename'].'.'.$testExt;
				if (file_exists($testFile))
				{
					$thExt = $testExt;
					continue;
				}
			}
						
			if ($thExt)
			{
				$info['imgthumbpath'] = $eleInfo['dirname'].'/'.AUTOGAL_THUMBPREFIX.$eleInfo['basename'].'.'.$thExt; 
				$eleDirName = dirname($info['element']); 
				if ($eleDirName == '.') $eleDirName = '';
				$info['imgthumburl'] = AutoGal_GetImageURL(($eleDirName ? "$eleDirName/" : '').AUTOGAL_THUMBPREFIX.$eleInfo['basename'].'.'.$thExt); 
				$info['imgthumbhtml'] = "<img src=\"".$info['imgthumburl']."\" alt=\"".$info['title']."\" style='border:none' />"; 
			}
			else
			{
				$info['imgthumbpath'] = realpath(AutoGal_GetUnavailThumb($info['element'])); 
				$info['imgthumburl'] = AutoGal_GetUnavailThumb($info['element']); 
				$info['imgthumbhtml'] = "<img src=\"".$info['imgthumburl']."\" alt=\"".$info['title']."\" style='border:none' />"; 
				$info['imgthumbunavail'] = true;
			}
		}
		else
		{
			$eleDirName = dirname($info['element']); 
			if ($eleDirName == '.') $eleDirName = '';
			
			// THUMBNAILS FOR IMAGES
			$info['imgthumbpath'] = $eleInfo['dirname'].'/'.AUTOGAL_THUMBPREFIX.$eleInfo['basename']; 
			$info['imgthumburl'] = AutoGal_GetImageURL(($eleDirName ? "$eleDirName/" : '').AUTOGAL_THUMBPREFIX.$eleInfo['basename']); 

			if (file_exists($info['imgthumbpath']))
			{
				$info['imgthumbhtml'] = "<img src=\"".$info['imgthumburl']."\" alt=\"".$info['title']."\" style='border:none' />";
			}
			else
			{
				if (AUTOGAL_AUTOTHUMB)
				{
					$info['imgthumbhtml'] = "<img src=\"".AUTOGAL_RESIZE."?img=".rawurlencode($info['element'])."\" alt=\"".$info['title']."\" style='border:none' />";
				}
				else
				{
					$unavailThumb = AutoGal_GetUnavailThumb($info['element']);
					$info['imgthumbpath'] = realpath($unavailThumb );
					$info['imgthumburl'] = $unavailThumb;
					$info['imgthumbhtml'] = "<img src=\"".$unavailThumb."\" style='border:none' />";
				}
			}
			
			$info['imgpvpath'] = $eleInfo['dirname'].'/'.AUTOGAL_PREVIEWIMGPREFIX.$eleInfo['basename']; 
			$info['imgpvurl'] = AutoGal_GetImageURL(($eleDirName ? "$eleDirName/" : '').AUTOGAL_PREVIEWIMGPREFIX.$eleInfo['basename']); 
		}
                
        $info['gallery'] = AutoGal_GetImageGallery($info['element']);        
    }
	
    if ($info['num'] <= 0)    
        $info['sortby'] = strtolower($info['title']);
    else    
        $info['sortby'] = $info['num'];
	
	//print $info['element']." = ".$info['sortby']."<br />";
	
	#$info['url'] = str_replace('//', '/', $info['url']);
	
	$g_eleInfoCache[$element] = $info;
    return ($info);
}

function AutoGal_GetElementXMLInfo($absPath)
{
	$debug = false;
	
	if ($debug) print "AutoGal_GetElementXMLInfo: $absPath<br />";
	if (AUTOGAL_USEXMLMETA)
	{
		$info['xmlread'] = true;
		
		if ($absPath == AutoGal_GetAbsGalPath())
		{
			$info['xmlfile'] = realpath(AUTOGAL_CONFIGDIR)."/Gallery.xml";
		}
		else
		{
			$info['xmlfile'] = $absPath.".xml";
		}
		
		$info['xmlok'] = false;
		$info['xmlfileexists'] = false;

		if ($debug) print "AUTOGAL_XMLDEBUG: ".$info['xmlfile']."<br />";
		if ((file_exists($info['xmlfile']))&&(filesize($info['xmlfile']) > 0))
		{
			$info['xmlfileexists'] = true;
			require_once(AUTOGAL_XMLMETAHANDLER);
			
			if ($debug) print "require_once<br />";
			
			$xmlMeta = new AutoGal_CMeta;
			if ($xmlMeta->LoadFile($info['xmlfile']))
			{
				if ($debug) print "require_once<br />";
				
				$xmlInfo = $xmlMeta->GetMetaData();
				
				if ($debug) AutoGal_Dump($xmlInfo)."<br />";
				
				foreach ($xmlInfo as $field => $value)
				{
					$info["ismetafile_$field"] = true;
					$info[$field] = $value;
				}
				
				$info['xmlok'] = true;
			}
			else
			{
				if ($debug) print "ERROR: ".$xmlMeta->m_lastError."<br />";
				$info['xmlerror'] = $xmlMeta->m_lastError;
			}
			
			if ($info['submitbyusername'])
			{
				if ($info['submitbyuserid'] > 0)
				{
					$info['submituserlink'] = "<a href=\"".e_BASE."user.php?id.".$info['submitbyuserid']."\">".$info['submitbyusername']."</a>";
				}
				else
				{
					$info['submituserlink'] = $info['submitbyusername'];
				}
			}
		}
	}
	
	return $info;
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_SetElementInfo($eleInfo, $perms=0, &$error)
{
	if (AUTOGAL_USEXMLMETA)
	{
		if (!$perms) $perms = AUTOGAL_PERMSGALXML;
		
		if (!$eleInfo['xmlfile'])
		{
			$metaInfo = AutoGal_GetElementXMLInfo($eleInfo['abspath']);
			foreach ($metaInfo as $metaField => $metaValue)
			{
				$eleInfo[$metaField] = $metaValue;
			}
		}
		
		require_once(AUTOGAL_XMLMETAHANDLER);
		$xmlMeta = new AutoGal_CMeta;
		$xmlMeta->LoadFromGetMeta($eleInfo);
		if (!$xmlMeta->SaveFile($eleInfo['xmlfile'], $perms))
		{
			$error = $xmlMeta->GetLastError()."<br />".AutoGal_GetXMLSaveError($eleInfo);
			return false;
		}
		
		return true;
	}
	
	return false;
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetImageURL($element)
{
	if ((preg_match('/['.chr(127).'-'.chr(255).']/', $element))&&(!AutoGal_IsFlashVideo($element)))
	{
		return (AUTOGAL_SHOWIMAGE."?img=".rawurlencode($element));
	}
	else
	{
		return AUTOGAL_BASE.str_replace('//', '/', '/'.AUTOGAL_GALLERYDIR.'/'.$element);
	}
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetUploadLink($gallery)
{
	$eleInfo = AutoGal_GetElementInfo($gallery, true);
	return (check_class(AUTOGAL_REVUPLOADUC) || check_class($eleInfo['uploadclass']) ? "[<a href=\"".AUTOGAL_UPLOAD.($gallery ? "?gallery=".rawurlencode($gallery) : '')."\">".AUTOGAL_LANG_L19."</a>]" : '');
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetStatNewestLink()
{
	return (AUTOGAL_SHOWNEWESTLINK ? "[<a href=\"".AUTOGAL_STATNEWEST."\">".AUTOGAL_LANG_STAT_L12."</a>]" : '');
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetSearchLink($gallery)
{
	return (AUTOGAL_ENABLESEARCH ? "[<a href=\"".AUTOGAL_SEARCH.($gallery != '' ? '?gallery='.rawurlencode($gallery) : '')."\">".AUTOGAL_LANG_SEARCH_L0."</a>]" : '');
}

function AutoGal_GetLatestCommentsLink()
{
	return (AUTOGAL_DOLATESTCOMMS ? "[<a href=\"".AUTOGAL_LATESTCOMMSVIEW."\">".AUTOGAL_LANG_COMMENTS_L14."</a>]" : '');
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetAdminLink()
{
	return (ADMIN ? "[<a href=\"".AUTOGAL_CONFIG."\">".AUTOGAL_LANG_L23."</a>] " : '');
}

function AutoGal_GetBotLinks($imageGallery='', $showReview=true, $showNewest=true, $showMain=true, $showSearch=true, $showLatestComms=true)
{
	$botLinks = array();

	if ($showMain)
	{
		$botLinks[] = "[<a href=\"".AUTOGAL_AUTOGALLERY."\">".AUTOGAL_ROOTNAME.' '.AUTOGAL_LANG_L24."</a>]";
	}
	
	if ($showNewest)
	{
		$link = AutoGal_GetStatNewestLink();
		if ($link) $botLinks[] = $link;
	}
	
	if ($imageGallery !== false)
	{
		$link = AutoGal_GetUploadLink($imageGallery);
		if ($link) $botLinks[] = $link;
	}
	
	if ($showReview)
	{
		$link = AutoGal_GetReviewLink();
		if ($link) $botLinks[] = $link;
	}
	
	if ($showSearch)
	{
		$link = AutoGal_GetSearchLink($imageGallery);
		if ($link) $botLinks[] = $link;
	}
	
	if ($showLatestComms)
	{	
		$link = AutoGal_GetLatestCommentsLink();
		if ($link) $botLinks[] = $link;
	}
	
	$link = AutoGal_GetAdminLink();
	if ($link) $botLinks[] = $link;
	
	return $botLinks;
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetVersion()
{
	$sqlVer = new db;
	$sqlVer->db_Select("plugin", "plugin_version", "plugin_name='Auto Gallery'");
	list($autoGalVer) = $sqlVer->db_Fetch();
	
	return $autoGalVer;
}

/*****************************************************************************************************
 * 
 ****************************************************************************************************/
function AutoGal_GetEmailLink($image)
{
	return (AUTOGAL_SHOWEMAILTOFRIEND ? "<b><a href=\"".AUTOGAL_EMAILTOFRIEND."?img=".rawurlencode($image).(AUTOGAL_SHOWINNEWWINDOW ? "&newwindow=1" : '')."\">".AUTOGAL_LANG_L20."</a></b>" : '');
}

function AutoGal_CheckResizeMethod(&$text)
{
	global $pref;
	
	$mode = ($pref['resize_method'] ? $pref['resize_method'] : "gd2");
	
    $renderTop = 0;
    if (preg_match("/^gd\d/", $mode))
    {
        if (!extension_loaded('gd')) 
		{
			$text = "<font color='red'><b>".AUTOGAL_LANG_L44."</b></font> ".str_replace('[MODE]', $mode, AUTOGAL_LANG_L45)." (<a href='".e_ADMIN."image.php'>".AUTOGAL_LANG_L46."</a>)";
			return false;
		}
		else
		{
			if (!function_exists('gd_info')) 
			{
				$text = "<font color='red'><b>".AUTOGAL_LANG_L44."</b></font> ".str_replace('[MODE]', $mode, AUTOGAL_LANG_L47)." (<a href='".e_ADMIN."image.php'>".AUTOGAL_LANG_L46."</a>)";
				return false;
			}
			$gdInfo = gd_info();
			
			$text = "<table colspan='2' class='border' cellpadding='3'><tr><td colspan='2' class='forumheader'><a href='http://www.boutell.com/gd/faq.html'>GD</a> ".AUTOGAL_LANG_L48."</td></tr>\n";
			foreach ($gdInfo as $gdField => $gdValue)
			{
				if (preg_match("/Support$/", $gdField))
				{
					if ($gdValue == 1) 
						$gdValue = 'Yes'; 
					elseif ($gdValue == '') 
						$gdValue = 'No';
				}
				$text .= "<tr><td class='forumheader3'><b>$gdField:</b></td><td class='forumheader3'>$gdValue</td></tr>\n";
			}
			$text .= "</table>";
			
			return true;
		}
    }
    elseif($mode == 'ImageMagick')
    {
        $imPath = $pref['im_path'];
        
		if ($imPath)
		{
			$currDir = getcwd();
			chdir($imPath);
		}
	
		$cmd = "convert -version";
		$IMVersion = shell_exec($cmd);
		
		if ($imPath) chdir($currDir);    
		
		if (!preg_match("/ImageMagick/", $IMVersion))
		{
			$text = "<b>".AUTOGAL_LANG_L44."</b> ".str_replace('[CMD]', "$cmd", str_replace('[MODE]', $mode, AUTOGAL_LANG_L49))." (<a href='".e_ADMIN."image.php'>".AUTOGAL_LANG_L46."</a>)";
			return false;
		}
		else
		{
			$IMVersion = str_replace("\n", "<br />", $IMVersion);
			$IMVersion = preg_replace("/(\<br\>)*Usage\:.+$/mi", "", $IMVersion);
			$IMVersion = preg_replace("/(\<br\>)+$/mi", "", $IMVersion);

			$text = "
			<table class='border' cellpadding='3'><tr><td class='forumheader'><a href='http://imagemagick.org'>ImageMagick</a> ".AUTOGAL_LANG_L48."</td></tr>
			<tr><td class='forumheader3'>$IMVersion</td></tr>
			</table>";
		}
		
		return true;
    }
	else
	{
		$text = "<b>".AUTOGAL_LANG_L44."</b> ".AUTOGAL_LANG_L50." (<a href='".e_ADMIN."image.php'>".AUTOGAL_LANG_L46."</a>)"; 
		return false;
	}
}

function AutoGal_GetLatestFiles($maxFiles=10, $start=0, &$totalFiles)
{
	$eles = AutoGal_GetGalleryElements('', true, (AUTOGAL_SORTNEWCREATION ? 'ctime' : 'mtime')); 
	$files = $eles['files'];
	$totalFiles = count($files);
	$latestFiles = array();
	
	for ($fileIndex = $start; $fileIndex < $totalFiles; $fileIndex ++)
	{
		$latestFiles[] = $files[$fileIndex];
		if (count($latestFiles) >= $maxFiles) break;
	}

	return $latestFiles;
}

function IsBadUploadDirPerms()
{
	if (AUTOGAL_CHMODWARNOFF) return;
	
	if (!file_exists(AUTOGAL_UPLOADDIRABS))
	{
		return "<div style='text-align:center'><font color='red'><b>".AUTOGAL_LANG_L51."</b></font> ".str_replace('[DIR]', AUTOGAL_UPLOADDIRABS, AUTOGAL_LANG_L52)."</div>";
	}
	
	$filePerms = fileperms(AUTOGAL_UPLOADDIRABS);
	$filePermsNum = substr(sprintf('%o', $filePerms), -3);
	$filePermsStr = AutoGal_FormatFilePerms($filePerms) . " ($filePermsNum)";
	if ($filePermsNum <> AUTOGAL_PERMSUPLDIR)
	{
		return "<div style='text-align: center'><font color='red'><b>".AUTOGAL_LANG_UPLOAD_L22." </b></font>".str_replace("[PERMS]", $filePermsStr, str_replace("[DIR]", AUTOGAL_UPLOADDIRABS, AUTOGAL_LANG_UPLOAD_L21))."<br />".AUTOGAL_LANG_L53."$filePermsStr</div>";
	}
	
	return "";
}

function IsBadGalleryDirPerms($gallery='')
{
	if (AUTOGAL_CHMODWARNOFF) return;
	
	$absPath = AutoGal_GetAbsGalPath($gallery);
	
	if (!file_exists($absPath))
	{
		return "<div style='text-align:center'><font color='red'><b>".AUTOGAL_LANG_L51."</b></font>".str_replace('[DIR]', $gallery, AUTOGAL_LANG_L54).(AUTOGAL_GALLERYDIR ? ' ('.AUTOGAL_GALLERYDIR.')' : '')."</div>";
	}
	
	$filePerms = fileperms($absPath);
	$filePermsNum = substr(sprintf('%o', $filePerms), -3);
	$filePermsStr = AutoGal_FormatFilePerms($filePerms) . " ($filePermsNum)";
	if ($filePermsNum <> AUTOGAL_PERMSGALDIR)
	{
		return "<div style='text-align:center'><font color='red'><b>".AUTOGAL_LANG_L51."</b></font>".str_replace('[DIR]', $gallery, str_replace('[PERMS]', AUTOGAL_PERMSGALDIR, AUTOGAL_LANG_L55))."<br />".AUTOGAL_LANG_L53."$filePermsStr</div>";
	}
	
	return "";
}

function IsBadLogDirPerms()
{
	if (AUTOGAL_CHMODWARNOFF) return;
	
	if (!file_exists(realpath(AUTOGAL_LOGDIR)))
	{
		return "<div style='text-align:center'><font color='red'><b>".AUTOGAL_LANG_L51."</b></font>".str_replace('[DIR]', AUTOGAL_LOGDIR, AUTOGAL_LANG_L56)."</div>";
	}
	
	$filePerms = fileperms(realpath(AUTOGAL_LOGDIR));
	$filePermsNum = substr(sprintf('%o', $filePerms), -3);
	$filePermsStr = AutoGal_FormatFilePerms($filePerms) . " ($filePermsNum)";
	if ($filePermsNum <> AUTOGAL_PERMSLOGDIR)
	{
		return "<div style='text-align:center'><font color='red'><b>".AUTOGAL_LANG_L51."</b></font>".str_replace('[DIR]', AUTOGAL_LOGDIR, str_replace('[PERMS]', AUTOGAL_PERMSLOGDIR, AUTOGAL_LANG_L57))."<br />".AUTOGAL_LANG_L53."$filePermsStr</div>";
	}
	
	return "";
}

function AutoGal_GetNewWindowHeader($title, $headExtra, $bodyClass, $bodyStyle)
{
	return 
	"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n".
	"<html xmlns=\"http://www.w3.org/1999/xhtml\">\n".
	"<head>\n".
	($title ? "<title>".$title."</title>" : '').
	"<link rel='stylesheet' href='".e_FILE."e107.css' type='text/css' />\n".
	"<link rel='stylesheet' href='".THEME."style.css' type='text/css' />\n".
	"<meta http-equiv='Content-Type' content='text/html; charset=".AUTOGAL_CHARSET."' />\n".
	"<meta http-equiv='content-style-type' content='text/css' />\n".
	($headExtra ? "$headExtra\n" : '').
	"<script type='text/javascript' src='".e_FILE."e107.js'></script>\n".
	"</head>\n".
	"<body".($bodyClass ? " class='$bodyClass'" : '').($bodyStyle ? " style='$bodyStyle'" : '').">\n";
}

function AutoGal_GetNewWindowFooter()
{
	return "
	</body>
	</html>";
}

function AutoGal_IncrementViewHits($eleInfo)
{
	global $ns;
	
	if (!AUTOGAL_USEXMLMETAVHITS) return $eleInfo;

	$eleInfo['viewhits'] ++;
	if (!AutoGal_SetElementInfo($eleInfo, 0, $errMsg))
	{	
		$ns -> tablerender(AUTOGAL_LANG_XML_L4, $errMsg);
	}
	
	return $eleInfo;
}

function AutoGal_GetXMLSaveError($eleInfo)
{
	if ($eleInfo['isroot'])
	{
		$needPermsDir = AUTOGAL_PERMSCFGDIR;
		$needPermsXml = AUTOGAL_PERMSCFGXML;
		
	}
	else
	{
		$needPermsDir = AUTOGAL_PERMSGALDIR;
		$needPermsXml = AUTOGAL_PERMSGALXML;
	}
	
	return AutoGal_XMLFilePermsError($eleInfo['xmlfile'], $needPermsXml, $needPermsDir);
}

function AutoGal_XMLFilePermsError($file, $needPermsXml=AUTOGAL_PERMSCFGXML, $needPermsDir=AUTOGAL_PERMSCFGDIR)
{
	if (!$needPermsXml) $needPermsXml = AUTOGAL_PERMSCFGXML;
	if (!$needPermsDir) $needPermsDir = AUTOGAL_PERMSCFGDIR;
		
	$xmlPerms = fileperms($file);
	$xmlPerms = substr(sprintf('%o', $xmlPerms), -3);
	
	$dirPerms = fileperms(dirname($file));
	$dirPerms = substr(sprintf('%o', $dirPerms), -3);
	
	if ($dirPerms != $needPermsDir)
	{
		$errMsgLang = AUTOGAL_LANG_XML_L2;
		$errMsgLang = str_replace("[DIR]", dirname($file), $errMsgLang);
		$errMsgLang = str_replace("[CURPERMS]", $xmlPerms, $errMsgLang);
		$errMsgLang = str_replace("[NEEDPERMS]", $needPermsXml, $errMsgLang);
	}
	elseif ($xmlPerms != $needPermsXml)
	{
		$errMsgLang = AUTOGAL_LANG_XML_L3;
		$errMsgLang = str_replace("[FILE]", $file, $errMsgLang);
		$errMsgLang = str_replace("[CURPERMS]", $xmlPerms, $errMsgLang);
		$errMsgLang = str_replace("[NEEDPERMS]", $needPermsXml, $errMsgLang);
	}
	
	$errMsg = AUTOGAL_LANG_XML_L1.'<br /><br />'.($errMsgLang ? $errMsgLang : $file);
	
	return $errMsg;
}

function AutoGal_IncrementEmailHits($eleInfo)
{
	global $ns;
	
	if (!AUTOGAL_USEXMLMETAEHITS) return $eleInfo;
	
	$eleInfo['emailhits'] ++;
	if (!AutoGal_SetElementInfo($eleInfo, 0, $errMsg))
	{
		$ns -> tablerender(AUTOGAL_LANG_XML_L4, $errMsg);
	}
	
	return $eleInfo;
}

function AutoGal_MetaDeleteComments($eleInfo, $IDs, &$newEleInfo)
{
	$comID = 0;
	while ($comID < count($IDs))
	{
		unset($eleInfo['comments'][$IDs[$comID]]);
		$comID ++;
	}

	if (!AutoGal_SetElementInfo($eleInfo, 0, $errMsg))
	{
		$text = $errMsg;
	}
	else
	{
		$text = AUTOGAL_LANG_COMMENTS_L11;
		$newEleInfo = $eleInfo;
		
		if (AUTOGAL_DOLATESTCOMMS) 
		{
			if (!AutoGal_RegenLatestComments($errMsg))
			{
				$text .= "<br />$errMsg";
			}
		}
	}
	
	return $text;
}

function AutoGal_RenderComments($eleInfo)
{
	global $ns;
	
	$isAdmin = check_class($eleInfo['adminclass']);
	$userCanPost = ($eleInfo['type'] == 'Gallery' ? check_class($eleInfo['gcommentclass']) : check_class($eleInfo['mcommentclass']));
	$text = "<form name='AutoGal_Comments' method='post'>";
		
	if ((!AUTOGAL_USEXMLMETA)||(!AUTOGAL_USEXMLMETACOMS)) return $eleInfo;
		
	if (($_POST['AutoGal_DeleteComments'])&&($isAdmin))
	{
		$comIDs = array();
		for ($commID = 0; $commID < count($eleInfo['comments']); $commID ++)
		{
			if ($_POST["AutoGal_CommentDelete_$commID"])
			{
				$comIDs[] = $commID;
			}
		}
		
		if (count($comIDs) <= 0)
		{
			$ns -> tablerender(AUTOGAL_LANG_COMMENTS_L10, "<div style='text-align:center'><b>".AUTOGAL_LANG_COMMENTS_L12."</b></div>");
		}
		else
		{
			$ns -> tablerender(AUTOGAL_LANG_COMMENTS_L10, AutoGal_MetaDeleteComments($eleInfo, $comIDs, $eleInfo));
		}
	}
	elseif ($_POST['AutoGal_SumbitComment'])
	{
		$commOK = false;
		$commentText = stripslashes($_POST['AutoGal_CommentText']);
		
		if (!$commentText)
		{
			$ns -> tablerender(AUTOGAL_LANG_COMMENTS_L2, "<div style='text-align:center'><b>".AUTOGAL_LANG_COMMENTS_L3."</b></div>");
		}
		elseif(trim($commentText) == trim($eleInfo['comments'][count($eleInfo['comments']) - 1]['text']))
		{
			$ns -> tablerender(AUTOGAL_LANG_COMMENTS_L2, "<div style='text-align:center'><b>".AUTOGAL_LANG_COMMENTS_L4."</b></div>");
		}
		else if (!$userCanPost)
		{
			$ns -> tablerender(AUTOGAL_LANG_COMMENTS_L2, "<div style='text-align:center'><b>".AUTOGAL_LANG_COMMENTS_L5."</b></div>");
		}
		else
		{
			if (USER)
			{
				$newComment['authorid'] = USERID;
				$newComment['authorusername'] = USERNAME;
				$commOK = true;
			}
			else
			{
				$newComment['authorid'] = 0;
				$newComment['authorusername'] = $_POST['AutoGal_CommentAuthor'];
				
				if (!$newComment['authorusername'] || strlen($newComment['authorusername']) > 30)
				{
					$ns -> tablerender(AUTOGAL_LANG_COMMENTS_L2, "<div style='text-align:center'><b>".AUTOGAL_LANG_COMMENTS_L6."</b></div>");
				}
				else
				{
					$commOK = true;
				}
			}
		}
		
		if ($commOK)
		{
			$newComment['text'] = $commentText;
			$newComment['date'] = time();
			
			$eleInfo['comments'][] = $newComment;
		
			if (AutoGal_SetElementInfo($eleInfo, 0, $errMsg))
			{
				$ns -> tablerender(AUTOGAL_LANG_COMMENTS_L2, "<div style='text-align:center'><b>".AUTOGAL_LANG_COMMENTS_L9."</b></div>");
				usort($eleInfo['comments'], "AutoGal_CompareComments");
				
				if (AUTOGAL_DOLATESTCOMMS)
				{
					# SAVE TO THE LATEST COMMENT XML FILE
					require_once(AUTOGAL_LTSTCOMSHANDLER);
					$lComms = new AutoGal_LatestComms(AUTOGAL_MAXLATESTCOMMS);
					$lComms->LoadFile(AUTOGAL_LATESTCOMMSXML);
					
					$addComment = $newComment;
					$addComment['element'] = $eleInfo['element'];
					
					$lComms->AddCommentArray($addComment);
					
					if (!$lComms->SaveFile(AUTOGAL_LATESTCOMMSXML, AUTOGAL_PERMSCFGXML))
					{
						$errMsg = $lComms->GetLastError()."<br />".AutoGal_XMLFilePermsError(AUTOGAL_LATESTCOMMSXML);
						$ns->tablerender(AUTOGAL_LANG_XML_L4, "<div style='text-align:center'>$errMsg</div>");
					}
				}
			}
			else
			{
				$ns -> tablerender(AUTOGAL_LANG_COMMENTS_L2, "<div style='text-align:center'>$errMsg</div>");
			}
		}
	}
	
	$numComments = 0;
	foreach ($eleInfo['comments'] as $commentI => $comment)
	{
		if (AUTOGAL_COMMENTBBCODE)
		{
			$commentText = AutoGal_DoBBCode($comment['text']);
		}
		else
		{
			$commentText = $comment['text'];
			$commentText = nl2br($commentText);
			$commentText = htmlspecialchars($commentText);
		}
	
		$text .= "
		<div class='spacer' style='text-align:center'>
		<table class='fborder' width='97%'>
		<tr>
			<td class='fcaption'".($isAdmin ? " colspan='2'" : '')."><b>".($comment['authorid'] == 0 ? $comment['authorusername'].AUTOGAL_LANG_COMMENTS_L8 : "<a href=\"".e_BASE."user.php?id.".$comment['authorid']."\">".$comment['authorusername']."</a>")."</b> @ ".strftime("%A %d %B %Y - %H:%M:%S", $comment['date'])."</td>
		</tr>
		<tr>".
			($isAdmin ?	"<td class='forumheader3' style='vertical-align:top'><input type='checkbox' name='AutoGal_CommentDelete_$commentI'></td>" : '').
			"<td class='forumheader3' style='vertical-align:top;width:100%'>$commentText</td>
		</tr>
		</table>
		</div>";
		
		$numComments ++;
	}
	
	$text .= "
	<br />
	<div class='spacer' style='text-align:center'>".
	
	($userCanPost ? "
	<table><tr><td style='text-align:left'>
	".(USER ? '' : AUTOGAL_LANG_COMMENTS_L7." <input name='AutoGal_CommentAuthor' class='tbox' size='30' maxlength='30'><br />")."
	<textarea name='AutoGal_CommentText' class='tbox' cols='70' rows='6'></textarea><br />
	</td></tr></table>
	<input type='submit' name='AutoGal_SumbitComment' class='button' value='".AUTOGAL_LANG_COMMENTS_L2."'>" : '').
	(($isAdmin)&&($numComments > 0) ? "<input type='submit' name='AutoGal_DeleteComments' class='button' value='".AUTOGAL_LANG_COMMENTS_L10."'>" : '')."<br />
	<span class='smalltext'>".(AUTOGAL_COMMENTBBCODE ? AUTOGAL_LANG_COMMENTS_L13 : '')."</span>
	</form>
	</div>";
	
	if (!(($numComments <= 0)&&(!$userCanPost)))
	{
		$ns -> tablerender(AUTOGAL_LANG_COMMENTS_L1. " ($numComments)", $text);
	}
	
	return $eleInfo;
}

function AutoGal_RenderDescription($eleInfo)
{
	global $ns;
	
	if ($eleInfo['description'])
	{
		$ns -> tablerender(str_replace("[TYPE]", $eleinfo['filetypetitle'], AUTOGAL_LANG_DESCRIPTION_L1), AutoGal_DoBBCode($eleInfo['description']));
	}
}

/*****************************************************************************************************
 * MAY SEEM STUPID NOW, BUT THIS IS SO I CAN SEPERATE FROM e107 EASILY
 ****************************************************************************************************/
function AutoGal_DoBBCode($text)
{
	$sql = new db;
	$sql->db_Select("core", "e107_value", "e107_name='e107'");
	list($e107Fields) = $sql->db_Fetch();
	$e107Fields = unserialize($e107Fields);
	$e107Ver = $e107Fields['e107_version'];
		
	if (preg_match("/0\.7/", $e107Ver))
	{
		$text = AutoGal_DoBBCode07($text);
	}
	elseif (preg_match("/0\.6/", $e107Ver))
	{
		if (class_exists(textparse))
		{
			global $tpa;
			if (!defined($tpa)) $tpa = new textparse;
			
			if (function_exists($tpa->bbcode))
			{
				$text = $tpa->bbcode($text);
			}
			else
			{
				$text = AutoGal_DoBBCode07($text);
			}
		}
		else
		{
			$text = AutoGal_DoBBCode07($text);
		}
	}
		
	return $text;
}

function AutoGal_DoBBCode07($text)
{
	require_once(e_HANDLER."bbcode_handler.php");
	$bbHandler = new e_bbcode;
	$text = htmlspecialchars($text);
	$text = $bbHandler->parseBBCodes($text);
	$text = nl2br($text);
	
	return $text;
}

function AutoGal_IsAdminMode()
{
	global $g_isAdmin;
	
	if (!$g_isAdmin) return 0;
	
	$oneMonth = time() + (60 * 60 * 24 * 30); 
	
	if (isset($_GET['adminmode']))
	{
		if ($_GET['adminmode'] == 1)
		{
			setcookie('ag_adminmode', 1, $oneMonth); 
			return 1;
		}
		else
		{
			setcookie('ag_adminmode', 0, 0); 
			return 0;
		}
	}
	
	if (isset($_COOKIE['ag_adminmode']))
	{
		return $_COOKIE['ag_adminmode'];
	}
	
	return 0;
}

function AutoGal_RenderAdminMode($eleInfo)
{
	global $g_isAdminMode;
	global $g_startImage;
	global $g_startImage;
	
	$getVarPrefix = '&';
	if ($eleInfo['isroot']) $getVarPrefix = '?';
	
	if ($g_isAdminMode)
	{
		$text = "[<a href=\"".$eleInfo['link'].$getVarPrefix."adminmode=0\">".AUTOGAL_LANG_ADMIN_MODE_1."</a>]";
	}
	else
	{
		$text = "[<a href=\"".$eleInfo['link'].$getVarPrefix."adminmode=1\">".AUTOGAL_LANG_ADMIN_MODE_0."</a>]";
	}
	
	return $text;
}

function AutoGal_RenderRating($eleInfo)
{
	if (!AUTOGAL_USEXMLMETARATINGS) return $eleInfo;
	if ($eleInfo['type'] != 'Image') return $eleInfo;
	
	global $ns;
		
	if (AUTOGAL_RATEIFRAME)
	{
		$text = "
		<iframe src=\"".AUTOGAL_RATING."?show=".rawurlencode($eleInfo['element'])."\" width='100%' frameborder='0' scrolling='no' height='".AUTOGAL_RATEIFRAMEHEIGHT."'>
		</iframe>";
		
		print $text;
	}
	else
	{
		$text = AutoGal_RatingHTML($eleInfo);
		$ns->tablerender(str_replace('[TYPE]', $eleInfo['filetypetitle'], AUTOGAL_LANG_RATING_L1), $text);
	}
	
	return $eleInfo;
}

function AutoGal_RatingHTML(&$eleInfo, $skipNew=0)
{
	if (!AUTOGAL_USEXMLMETARATINGS) return;
	if ($eleInfo['type'] != 'Image') return;
		
	global $ns;
	
	if (!$eleInfo['xmlread'])
	{
		$eleInfo = AutoGal_GetElementInfo($eleInfo['element'], true);
	}
	
	# GET RATING INFO
	$ratingRecs = $eleInfo['ratings'];
	$numRatings = count($ratingRecs);
	$userCanRate = check_class(AUTOGAL_RATECLASS);
	$userHasRated = 0;
	
	if (USER)
	{
		$rateUsername = USERNAME;
		$rateUserID = USERID;
	}
	else
	{
		$rateUsername = $_SERVER['REMOTE_ADDR'];
		$rateUserID = 0;
	}
	
	foreach ($ratingRecs as $rec)
	{
		$total += $rec['mark'];
		
		if ($rateUserID == 0)
		{
			if ($rateUsername == $rec['username'])
			{
				$userHasRated = 1;
			}
		}
		else
		{
			if ($rateUserID == $rec['userid'])
			{
				$userHasRated = 1;
			}
		}
	}
		
	$rating = number_format($total / $numRatings, 2);
	$rateImages = AutoGal_RatingImages($rating);
	
	$rateMsg = AUTOGAL_LANG_RATING_L5;
	$rateMsg = str_replace('[TYPE]', $eleInfo['filetypetitle'], $rateMsg);
	$rateMsg = str_replace('[NUMVOTES]', $numRatings, $rateMsg);
	$rateMsg = str_replace('[RATING]', $rating, $rateMsg);
	$rateMsg = str_replace('[MAXRATING]', number_format(AUTOGAL_MAXRATE, 2), $rateMsg);
		
	# PROCESS RATING
	if (($_POST['ag_dorating'])&&(!$skipNew))
	{
		$newRatingVal = $_POST['ag_rating'];
		
		if ((!$userHasRated)&&($newRatingVal <= AUTOGAL_MAXRATE)&&($newRatingVal >= 1)&&($userCanRate))
		{
			$newRating['mark'] = $newRatingVal;
			$newRating['userid'] = $rateUserID;
			$newRating['username'] = $rateUsername;
			$newRating['date'] = time();
			
			$eleInfo['ratings'][] = $newRating;
			if (!AutoGal_SetElementInfo($eleInfo, 0, $errMsg))
			{
				$ns -> tablerender(AUTOGAL_LANG_XML_L4, $errMsg);
			}
			return AutoGal_RatingHTML($eleInfo, 1);
		}
	}
		
	# RENDER RATING	
	$text .= "<div style='text-align:center'>";
	
	if (!$userCanRate)
	{
		if ($numRatings == 0)
		{
			$text .= AUTOGAL_LANG_RATING_L3;
		}
		else
		{
			$text .= $rateImages."<br /><span class='smalltext'>$rateMsg</span>";
		}
	}
	else
	{
		for ($rateNum = 1; $rateNum <= AUTOGAL_MAXRATE; $rateNum ++)
		{
			$rateSelectNums .= "<td style='text-align:center'><b>$rateNum</b></td>";
			$rateSelectOpts .= "<td style='text-align:center'><input type='radio' name='ag_rating' value='$rateNum'".($rateNum == ceil(AUTOGAL_MAXRATE / 2) ? " checked='checked'" : '')."></td>";
		}
		
		$rateSelect = "<form method='POST'>
		<table style='text-align:center'>
			<tr>$rateSelectNums<td rowspan='2' style='text-align:center;padding-left:5px'><input type='submit' name='ag_dorating' class='button' value='".AUTOGAL_LANG_RATING_L6."'></td></tr>
			<tr>$rateSelectOpts</tr>
		</table>
		<input type='hidden' name='show' value=\"".rawurlencode($eleInfo['element'])."\">
		</form>";

		if ($numRatings == 0)
		{
			$text .= AUTOGAL_LANG_RATING_L3."<br />".$rateSelect;
		}
		else if ($userHasRated)
		{
			$text .= $rateImages."<br /><span class='smalltext'>$rateMsg<br />".AUTOGAL_LANG_RATING_L2."</span>";
		}
		else
		{
			$text .= $rateImages."<br /><span class='smalltext'>$rateMsg</span><br />".$rateSelect;
		}
	}

	$text .= "</div>";
	
	return $text;
}

function AutoGal_RatingImages($ratingVal)
{
	for ($starCnt = 1; $starCnt <= AUTOGAL_MAXRATE; $starCnt ++)
	{
		if ($starCnt <= floor($ratingVal))
		{
			$text .= "<img src='".AUTOGAL_RATEIMAGEFULL."' style='border:0px;'>&nbsp;";
		}
		else if ($ratingVal >= ($starCnt - 0.5))
		{
			$text .= "<img src='".AUTOGAL_RATEIMAGEHALF."' style='border:0px;'>&nbsp;";
		}
		else
		{
			$text .= "<img src='".AUTOGAL_RATEIMAGEBLANK."' style='border:0px;'>&nbsp;";
		}
	}
	
	return $text;
}

function AutoGal_AddScoreGetUser($points)
{
	if (AUTOGAL_ARCADEUSEXMLTRACK)
	{
		require_once (AUTOGAL_ARCADEPLAYERS);
		$players = new AutoGal_ArcadePlayers(AUTOGAL_ARCADEPLAYERSXML);
		
		$errMsg .= "<b>XML Player Tracker</b><br />";
		if (file_exists(AUTOGAL_ARCADEPLAYERSXML))
		{
			if (!$players->Open())
			{
				$errMsg .= "XML player tracker file (".AUTOGAL_ARCADEPLAYERSXML.") open error: ".$players->LastError()."<br />";
				print $errMsg;
				exit;
			}
			
			$element = $players->PlayerElement(USERID);
			
			if (!$element)
			{
				$errMsg .= "No element for USERID=".USERID."<br />";
				print $errMsg;
				exit;
			}
		}
		else
		{
			$errMsg .= "Player tracker file \"".AUTOGAL_ARCADEPLAYERSXML."\" does not exist<br />";
			print $errMsg;
			exit;
		}
	}
	else
	{
		$referer = $_SERVER['HTTP_REFERER'];
		$element = AutoGal_GetEleFromUrl($referer, $error);
		
		if (!$element)
		{
			print AUTOGAL_LANG_ARCADE_L7.htmlspecialchars($referer)."<br />$error<br />".AUTOGAL_LANG_ARCADE_L9;
			exit;
		}
	}
	
	if ((!defined('USERID'))||(!defined('USERNAME')))
	{
		print AUTOGAL_LANG_ARCADE_L6."<br />";
		exit;
	}
	
	if (!AutoGal_AddScore($element, USERID, USERNAME, $points, $error))
	{
		$addScorErrMsg = AUTOGAL_LANG_ARCADE_L8;
		$addScorErrMsg = str_replace("[SCORE]", htmlspecialchars($points), $addScorErrMsg);
		$addScorErrMsg = str_replace("[ELEMENT]", htmlspecialchars($element), $addScorErrMsg);
		$errMsg .= "$addScorErrMsg<br /><br />$error<br />";
		print $errMsg;
		exit;
	}
	
	header("location: ".AUTOGAL_AUTOGALLERY."?show=".rawurlencode($element));
	exit;
}

function AutoGal_GetEleFromUrl($url, &$error)
{
	$server = $_SERVER['SERVER_NAME'];
	$uri = $_SERVER['REQUEST_URI'];
	
	$error = "<font face='courier new'>
	Server Name: ".htmlspecialchars($_SERVER['SERVER_NAME'])."<br />
	Request URI: ".htmlspecialchars($_SERVER['REQUEST_URI'])."<br />";
	
	# GET FROM SHOW VARIABLE
	$url = preg_replace("/^(http|https)\:\/\//", '', $url);
	$url = preg_replace("/^www\./", '', $url);
	$server = preg_replace("/^www\./", '', $server);
	
	$uriDir = dirname($uri);
	if (!preg_match("/^\//", $uriDir)) $uriDir = "/$uriDir";
	if (!preg_match("/\/$/", $uriDir)) $uriDir .= "/";
	
	$testRegex = preg_quote($server.$uriDir.basename(AUTOGAL_AUTOGALLERY), '/');
	if (preg_match("/^$testRegex(.*)$/i", $url, $bits))
	{
		$urlInfo = parse_url($url);
		parse_str($urlInfo['query'], $getVars);
		
		if ($getVars['show'])
		{	
			$error = '';
			$element = $getVars['show'];
			$element = rawurldecode($element);
			return $element;
		}
	}
	
	$error .= "<b>Show Var Test</b><br />Refferer: \"".htmlspecialchars($url)."\"<br />Regex: /^".htmlspecialchars($testRegex)."/i<br />";
	
	# GET FROM REFFERER URL PATH
	$galHttpDir = $_SERVER['SCRIPT_URI'];
	
	if ($galHttpDir)
	{
		$galHttpDir = preg_replace("/^(\w+)\:\/\//", '', $galHttpDir);
		$galHttpDir = preg_replace("/^www\./", '', $galHttpDir);
		$galHttpDir = dirname($galHttpDir);
	}
	else
	{
		$galHttpDir = $server.$uriDir;
	}
	
	if (!preg_match("/\/$/", $galHttpDir)) $galHttpDir .= "/";
	$galHttpDir .= AUTOGAL_GALLERYDIR;
	
	$galHttpDirClean = AutoGal_CleanPath($galHttpDir);
		
	if (preg_match("/^".preg_quote($galHttpDirClean, '/')."(.+)$/i", $url, $bits))
	{
		$element = $bits[1];
		$element = preg_replace("/^[\/]+/", '', $element);
		$element = rawurldecode($element);
		$error = '';
		return $element;
	}
	
	# NUP, NOTHING WORKED
	$error .= "
	<b>Path Test</b><br />
	URL: \"".htmlspecialchars($url)."\"<br />
	Script URI: \"".htmlspecialchars($_SERVER['SCRIPT_URI'])."\"<br />
	Gallery HTTP Dir: \"".htmlspecialchars($galHttpDir)."\"<br />
	Cleaned: \"".htmlspecialchars($galHttpDirClean)."\"<br />
	Gallery Dir: \"".htmlspecialchars(AUTOGAL_GALLERYDIR)."\"<br />".
	"<br />".
	"POST:<br />";
	foreach ($_POST as $var => $val)
	{
		$error .= htmlspecialchars($var)."=\"".htmlspecialchars($val)."\"";
	}
	
	$error .= "
	<br />
	SESSION:<br />";
	foreach ($_SESSION as $var => $val)
	{
		$error .= htmlspecialchars($var)."=\"".htmlspecialchars($val)."\"";
	}
	
	$error .= "
	<br />
	</font>
	No show variable in referrer/invalid refferer!";
	
		
	return '';
}

# RIPPED FROM http://au3.php.net/manual/en/function.realpath.php (bart at mediawave dot nl)
function AutoGal_CleanPath($path)
{
   $result = array();
   // $pathA = preg_split('/[\/\\\]/', $path);
   $pathA = explode('/', $path);
   if (!$pathA[0])
       $result[] = '';
   foreach ($pathA AS $key => $dir) {
       if ($dir == '..') {
           if (end($result) == '..') {
               $result[] = '..';
           } elseif (!array_pop($result)) {
               $result[] = '..';
           }
       } elseif ($dir && $dir != '.') {
           $result[] = $dir;
       }
   }
   if (!end($pathA))
       $result[] = '';
   return implode('/', $result);
}

function AutoGal_AddScore($element, $userID, $username, $points, &$error)
{
	if (!AUTOGAL_ARCTOPSCORES)
	{
		$error = "Arcade top scores not enabled";
		return 0;
	}
	
	if (!$element) 
	{
		$error = "Blank element '$element'";
		return 0;
	}
	
	if (!preg_match("/^[0-9]+$/", $userID))
	{
		$error = "User ID '$userID' not numeric";
		return 0;
	}
	
	if (!$username)
	{
		$error = "Username blank";
		return 0;
	}
	
	if (!preg_match("/^\-?[\.0-9]+$/", $points)) 
	{
		$error = "Points '$points' not numeric";
		return 0;
	}
	
	$eleInfo = AutoGal_GetElementInfo($element, true);
	if (!$eleInfo)
	{
		$error = "Invalid element '$element'";
		return 0;
	}
	
	if (!preg_match("/^(".AUTOGAL_ARCADEEXTS.")$/i", $eleInfo['extension'])) 
	{
		$error = "Not arcade extension '".$eleInfo['extension']."' (allowed extensions: ".str_replce('|', ', ', AUTOGAL_ARCADEEXTS).")!";
		return 0;
	}
	
	$scores = $eleInfo['arcade']['topscores'];
	usort($scores, "AutoGal_CmpArcadeScores");
	$scores = array_slice($scores, 0, AUTOGAL_ARCMAXTOPSCORES);
	
	# Try and detect repeat score
	$lastUserScoreDate = 0;
	$lastUserScorePoints = 0;
	foreach ($scores as $score)
	{
		if (($score['username'] == $username)&&($score['date'] > $lastUserScoreDate))
		{
			$lastUserScoreDate = $score['date'];
			$lastUserScorePoints = $score['points'];
		}
	}
	
	if ($lastUserScorePoints == $points)
	{
		return 3;
	}
		
	$numScores = count($scores);
	if ($numScores > 0)
	{
		$lowestPoints = $scores[$numScores - 1]['points'];
		
		if (($points < $lowestPoints)&&($numScores >= AUTOGAL_ARCMAXTOPSCORES))
		{
			return 2;
		}
	}
	
	$scoreRec = array();
	$scoreRec['userid'] = $userID;
	$scoreRec['username'] = $username;
	$scoreRec['date'] = time();
	$scoreRec['points'] = $points;
	
	$scores[] = $scoreRec;
	usort($scores, "AutoGal_CmpArcadeScores");
	$scores = array_slice($scores, 0, AUTOGAL_ARCMAXTOPSCORES);
	
	$eleInfo['arcade']['topscores'] = $scores;
	
	if (!AutoGal_SetElementInfo($eleInfo, 0, $errMsg))
	{
		$error = $errMsg;
		return 0;
	}
	
	return 1;
}

function AutoGal_RenderTopScores($eleInfo)
{
	global $ns;
	if (!AUTOGAL_ARCTOPSCORES) return;
	if (!preg_match("/^(".AUTOGAL_ARCADEEXTS.")$/i", $eleInfo['extension'])) return;
	
	if (!$eleInfo['xmlread'])
	{
		$eleInfo = AutoGal_GetElementInfo($eleInfo['element'], true);
	}
	
	$scores = $eleInfo['arcade']['topscores'];
	
	if (count($scores) <= 0) return;
	
	usort($scores, "AutoGal_CmpArcadeScores");
	$scores = array_slice($scores, 0, AUTOGAL_ARCMAXTOPSCORES);
		
	$text = "
	<div style='text-align:center'>
	<table class='border'>
	<tr>
		<th class='forumheader' style='text-align:center'>".AUTOGAL_LANG_ARCADE_L2."</th>
		<th class='forumheader' style='text-align:center'>".AUTOGAL_LANG_ARCADE_L3."</th>
		<th class='forumheader' style='text-align:center'>".AUTOGAL_LANG_ARCADE_L5."</th>
		<th class='forumheader' style='text-align:center'>".AUTOGAL_LANG_ARCADE_L4."</th>
	</tr>";
	
	$rank = 1;
	foreach ($scores as $score)
	{
		$user = $score['username'];
		$userID = $score['userid'];
		$points = $score['points'];
		$date = strftime("%A %d %B %Y - %H:%M:%S", $score['date']);
		
		if ($userID > 0)
		{
			$user = "<a href=\"".e_BASE."user.php?id.$userID\">$user</a>";
		}
		
		$text .= "
		<tr>
			<td class='forumheader3' style='text-align:left'>$rank.</td>
			<td class='forumheader3' style='text-align:center'>$user</td>
			<td class='forumheader3' style='text-align:center'>$date</td>
			<td class='forumheader3' style='text-align:right'>$points</td>
		</tr>";
		
		$rank ++;
	}
	
	$text .= "
	</table>
	</div>";
	
	$ns -> tablerender(AUTOGAL_LANG_ARCADE_L1, $text);
}

function AutoGal_CmpArcadeScores($a, $b)
{
	if ($a['points'] == $b['points'])
	{
		if ($a['date'] == $b['date']) return 0;
				
		return ($a['date'] > $b['date']) ? -1 : 1;
	}
	
	return ($a['points'] > $b['points']) ? -1 : 1;
}

function AutoGal_ResizeImage($imgPath, $destPath, $width, $height, $keepAspect=null)
{
	global $pref;
	
	require_once(AUTOGAL_IMGMANIPHANDLER);
	
	if ($keepAspect == null) $keepAspect = AUTOGAL_KEEPASPECT;
	
	$mode = $pref['resize_method'];
	$imPath = $pref['im_path'];
	$gdim = new GDIM($mode, $imPath);
	
	if (!$gdim->resize($imgPath, $destPath, $width, $height, array('keepaspect' => $keepAspect)))
	{
		return $gdim->lastError();
	}
	
	return '';
}

function AutoGal_RegenLatestComments(&$error, $skipCommsTest=0)
{
	if ((!AUTOGAL_DOLATESTCOMMS)&&(!$skipCommsTest)) 
	{
		$error = "Comments not enabled";
		return;
	}
	
	require_once(AUTOGAL_LTSTCOMSHANDLER);
	
	# CLEAR CACHE
	global $g_eleInfoCache;
	$g_eleInfoCache = array();
	
	$lComms = new AutoGal_LatestComms(AUTOGAL_MAXLATESTCOMMS);
	$files = AutoGal_ListDirectory(AutoGal_GetAbsGalPath(''), "^(".AUTOGAL_THUMBPREFIX.'|'.AUTOGAL_DEFAULTIMAGE.')', false);      
	
	foreach ($files as $file)
	{
		$element = AutoGal_GetElement($file);
		$eleInfo = AutoGal_GetElementInfo($element, true);
		
		foreach ($eleInfo['comments'] as $comment)
		{
			$comment['element'] = $element;
			$lComms->AddCommentArray($comment);
		}
	}
	
	if (!$lComms->SaveFile(AUTOGAL_LATESTCOMMSXML, AUTOGAL_PERMSCFGXML))
	{
		$error = $lComms->GetLastError()."<br />".AutoGal_XMLFilePermsError(AUTOGAL_LATESTCOMMSXML);
		return false;
	}
	
	return true;
}

function AutoGal_UploadFile($name, $path)
{
	if (move_uploaded_file($_FILES[$name]['tmp_name'], $path))
	{
		if (!file_exists($path))
		{
			$msgs = str_replace('[DIR]', dirname($path), AUTOGAL_LANG_UPLOAD_L26);
			return array(0, $msgs);
		}
		else
		{
			return array(1, '');
		}
	}
	
	switch($_FILES[$name]['error'])
	{
		case UPLOAD_ERR_INI_SIZE: $msgs = AUTOGAL_LANG_UPLOAD_L9; break;
		case UPLOAD_ERR_FORM_SIZE: $msgs = AUTOGAL_LANG_UPLOAD_L10 . AUTOGAL_UPLOADMAXSIZE . " " . AUTOGAL_LANG_UPLOAD_L27; break;
		case UPLOAD_ERR_NO_TMP_DIR: $msgs = AUTOGAL_LANG_UPLOAD_L11; break;
		case UPLOAD_ERR_NO_FILE: $msgs = AUTOGAL_LANG_UPLOAD_L12; break;
		default: $msgs = "(".$_FILES[$name]['error'].") ".str_replace('[DIR]', dirname($path), AUTOGAL_LANG_UPLOAD_L26);
	}
	
	return array(0, $msgs);
}

function AutoGal_ClearDirCache($gallery, $silent=false)
{
	if (!AUTOGAL_DIRCACHEMETHOD) return true;
	
	global $ns;
	$galleryPath = AutoGal_GetAbsGalPath($gallery);
	$cachePath = str_replace('//', '/', $galleryPath.'/'.AUTOGAL_DIRCACHEFILE);
	
	if (!file_exists($cachePath)) return true;
	
	if (unlink($cachePath))
	{
		if (!$silent) $ns->tablerender(AUTOGAL_LANG_CACHE_L1, str_replace("[GALLERY]", htmlspecialchars($gallery), AUTOGAL_LANG_CACHE_L2));
		return true;
	}
	else
	{
		if (!$silent) $ns->tablerender(AUTOGAL_LANG_CACHE_L1, str_replace("[FILE]", htmlspecialchars($cachePath), str_replace("[GALLERY]", htmlspecialchars($gallery), AUTOGAL_LANG_CACHE_L2)));
		return false;
	}
}

function AutoGal_FileNavButtons($eleInfo, $showSlideShow=false, $showClose=false)
{
	$element = $eleInfo['element'];
	$gallery = $eleInfo['gallery'];
    $galEles = AutoGal_GetGalleryElements($gallery);
	$currIndex = AutoGal_GetImageIndex($element, $galEles);
	
	if (AUTOGAL_SHOWINNEWWINDOW) $shNewWin = "&newwindow=1";
	$start = (preg_match("/^[0-9]+$/", $_GET['start']) ? $_GET['start'] : 0);
	$startParam = ($start ? "&start=$start" : '');
    
    $firstFile = ($currIndex == 0 ? '' : $galEles['files'][0]);
    $lastFile = ($currIndex == (count($galEles['files']) - 1) ? '' : $galEles['files'][count($galEles['files']) - 1]);
    $prevFile = $galEles['files'][$currIndex - 1];
    $nextFile = $galEles['files'][$currIndex + 1];
	
    $firstFile = ($firstFile ? "<input type='button' title=\"".AUTOGAL_LANG_L12.$firstFile['title']."\" class='button' value='".AUTOGAL_LANG_L7."' onclick='javascript:window.location.href=\"".AUTOGAL_AUTOGALLERY."?show=".urlencode($firstFile['element'])."$startParam$shNewWin\"' />" : "<input type='button' value='".AUTOGAL_LANG_L7."' class='button' disabled='disabled' />");
    $prevFile = ($prevFile ? "<input type='button' title=\"Image: ".$prevFile['title']."\" class='button' value='".AUTOGAL_LANG_L8."' onclick='javascript:window.location.href=\"".AUTOGAL_AUTOGALLERY."?show=".urlencode($prevFile['element'])."$startParam$shNewWin\"' />" : "<input type='button' value='".AUTOGAL_LANG_L8."' class='button' disabled='disabled' />");
    $nextFile = ($nextFile ? "<input type='button' title=\"Image: ".$nextFile['title']."\" class='button' value='".AUTOGAL_LANG_L9."' onclick='javascript:window.location.href=\"".AUTOGAL_AUTOGALLERY."?show=".urlencode($nextFile['element'])."$startParam$shNewWin\"' />" : "<input type='button' value='".AUTOGAL_LANG_L9."' class='button' disabled='disabled' />");
    $lastFile = ($lastFile ? "<input type='button' title=\"Image: ".$lastFile['title']."\" class='button' value='".AUTOGAL_LANG_L10."' onclick='javascript:window.location.href=\"".AUTOGAL_AUTOGALLERY."?show=".urlencode($lastFile['element'])."$startParam$shNewWin\"' />" : "<input type='button' value='".AUTOGAL_LANG_L10."' class='button' disabled='disabled' />");
    $galleryLink = "<input type='button' title=\"".AUTOGAL_LANG_L13.AutoGal_GetTitle($gallery, 1, 'title')."\" class='button' value='".AUTOGAL_LANG_L11."' onclick='javascript:window.location.href=\"".AUTOGAL_AUTOGALLERY."?show=".urlencode($gallery)."&amp;$startParam$shNewWin\"' />";
	$closeWindow = "<input type='button' class='button' value='".AUTOGAL_LANG_L25."' onClick='javascript:window.close()'>";
	
	if (AUTOGAL_SLIDESENABLE)
	{
		if (AUTOGAL_SLIDESNEWWINDOW)
		{
			$slideShowURL = AUTOGAL_SLIDESHOW."?first=".rawurlencode($eleInfo['element']).$startParam;
			$slideOnClickJS = AutoGal_NewWindowJS('slide', $slideShowURL, 'slideshow');
		}
		else
		{
			$slideShowURL = AUTOGAL_SLIDESHOW."?first=".htmlspecialchars($eleInfo['element']).$startParam;
			$slideOnClickJS = "window.location.href=\"$slideShowURL\"";
		}
		
		$slideShow = "<input type='button' class='button' title=\"".AUTOGAL_LANG_SLIDESHOW_L2."\" value='".AUTOGAL_LANG_SLIDESHOW_L1."' onclick=\"javascript:$slideOnClickJS\" />";
	}
  
	$text = "$firstFile&#160;$prevFile&#160;".($showClose ? $closeWindow : $galleryLink).($showSlideShow ? "&#160;$slideShow" : '')."&#160;$nextFile&#160;$lastFile";
	return $text;
}

function AutoGal_FilePreviewHTML($eleInfo, $showFullImage, $showLimitMsg=true, $fullImgNewWin=AUTOGAL_LARGEIMGNEWWINDOW)
{
	global $ns;
	
	$fileSize = filesize($eleInfo['abspath']);
	$fileSizeName = array(" Bytes", " KB", " MB", " GB", " TB", " PB", " EB", " ZB", " YB");
	$fileSize = round($fileSize/pow(1024, ($i = floor(log($fileSize, 1024)))), 2) . $fileSizeName[$i];

	if ($eleInfo['isflash'])
	{
		$pvWidth = ($eleInfo['viewwidth'] ? $eleInfo['viewwidth'] : AUTOGAL_FLASHWIDTH);
		$pvHeight = ($eleInfo['viewheight'] ? $eleInfo['viewheight'] : AUTOGAL_FLASHHEIGHT);
		
		$text = '
		<object width="'.$pvWidth.'" height="'.$pvHeight.'">
			<param name="movie" value="'.$eleInfo['basename'].'">
			<param name=quality value=high>
			<param name=bgcolor value=#FFFFFF>
			<embed src="'.$eleInfo['url'].'" 
				width="'.$pvWidth.'" height="'.$pvHeight.'" align="" 
				name="'.$eleInfo['title'].'" 
				type="application/x-shockwave-flash"
				pluginspace="http://www.macromedia.com/go/getflashplayer">
			</embed>
		</object><br />'
		.AUTOGAL_LANG_L37.$fileSize.'<br />';
	}
	# THANKS TO EMMANUAEL FOR THE FLV PLAYER
	else if ($eleInfo['isflv'])
	{
		$url = urlencode($eleInfo['url']); # FF
		$target = AUTOGAL_FLVPLAYER.'?file='.$url.'&autoStart=false';
		
		$pvWidth = ($eleInfo['viewwidth'] ? $eleInfo['viewwidth'] : AUTOGAL_FLASHWIDTH);
		$pvHeight = ($eleInfo['viewheight'] ? $eleInfo['viewheight'] : AUTOGAL_FLASHHEIGHT);
				
		if (AUTOGAL_SHOWEMBEDLINK)
		{
			$embed = '<embed src="'.AUTOGAL_FLVPLAYER.'" width="'.$pvWidth.'" height="'.$pvHeight .'" name="'.$eleInfo['title'].'" type="application/x-shockwave-flash" pluginspace="http://www.macromedia.com/go/getflashplayer" flashvars="file='.AutoGal_ChangeToEmbed($eleInfo['url']).'&autoStart=false&lightcolor=0x996600&backcolor=0x000000&frontcolor=0xCCCCCC&repeat=true&logo='.AUTOGAL_FLVPLAYER_LOGO.'"></embed>';
			$inputEmbed ='<input name="ag_embedcode" type="text" value=\''.$embed.'\' readonly="true" onfocus=\'this.select();this.focus();\' onClick=\'this.select();this.focus();\'>';
			$inputLink  ='<input name="ag_embedlink" type="text" value=\''.$target.'\' readonly="true" onfocus=\'this.select();this.focus();\' onClick=\'this.select();this.focus();\'>';
			$embedHtmlCode = '<br /><table><tr><td><b>'.AUTOGAL_LANG_EMBED_L1.'</b></td><td>'.$inputEmbed.'</td><tr><tr><td><b>'.AUTOGAL_LANG_EMBED_L2.'</b></td><td>'.$inputLink.'</td></tr></table>';
		}
		else
		{
			$embedHtmlCode = '';
		}
		
		$text = '
		<object width="'.$pvWidth.'" height="'.$pvHeight.'">
			<param name="movie" value="'.$eleInfo['basename'].'">
			<param name="quality" value="high">
			<param name="bgcolor" value="#FFFFFF">
			<param name="flashvars" value="file='.$url.'&autoStart=false">
			<embed src="'.AUTOGAL_FLVPLAYER.'" 
				width="'.$pvWidth.'" height="'.$pvHeight.'"
				name="'.$eleInfo['title'].'" 
				type="application/x-shockwave-flash"
				pluginspace="http://www.macromedia.com/go/getflashplayer"
				flashvars="file='.$url.'&autoStart=false">
			</embed>
		</object><br />'
		.AUTOGAL_LANG_L37.$fileSize.$embedHtmlCode.'<br />';
	}
	else if ($eleInfo['isquicktime'])
	{
		$pvWidth = ($eleInfo['viewwidth'] ? $eleInfo['viewwidth'] : AUTOGAL_MOVIEWIDTH);
		$pvHeight = ($eleInfo['viewheight'] ? $eleInfo['viewheight'] : AUTOGAL_MOVIEHEIGHT);
		
		$text = '
		<object classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B" 
			width="'.$pvWidth.'" height="'.($pvHeight + 15).'" 
			codebase="http://www.apple.com/qtactivex/qtplugin.cab">
			<param name="src" value="'.$eleInfo['url'].'">
			<param name="autoplay" value="true">
			<param name="controller" value="true">
			<param name="loop" value="true">
			<embed src="'.$eleInfo['url'].'" 
				width="'.$pvWidth.'" height="'.($pvHeight + 15).'" 
				autoplay="true" 
				controller="true" 
				loop="true" 
				pluginspage="http://www.apple.com/quicktime/download/">
			</embed>
        </object><br />'
		.AUTOGAL_LANG_L37.$fileSize.'<br />';
	}
	else if ($eleInfo['iswinmedia'])
	{
		if ($eleInfo['isaudio'])
		{
			$pvWidth = 300;
			$pvHeight = 46;
		}
		else
		{
			$pvWidth = ($eleInfo['viewwidth'] ? $eleInfo['viewwidth'] : AUTOGAL_MOVIEWIDTH);
			$pvHeight = ($eleInfo['viewheight'] ? $eleInfo['viewheight'] : AUTOGAL_MOVIEHEIGHT);
		}
		
		$src = $eleInfo['url'];
		
		$text = '
		<table border="0" cellpadding="0" align="center" width="'.$pvWidth.'" height="'.$pvHeight.'">
        <tr><td>
			<object id="mediaPlayer"
				classid="CLSID:22d6f312-b0f6-11d0-94ab-0080c74c7e95" 
				width="'.$pvWidth.'" height="'.$pvHeight.'" 
				codebase="http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=5,1,52,701"
				standby="'.AUTOGAL_LANG_L27.'" type="application/x-oleobject">
				<param name="fileName" value="'.$src.'">
				<param name="animationatStart" value="true">
				<param name="transparentatStart" value="true">
				<param name="autoStart" value="true">
				<param name="showControls" value="true">
				<param name="loop" value="true">
				<embed src="'.$src.'" 
					type="application/x-mplayer2"
					width="'.$pvWidth.'" height="'.$pvHeight.'"
					pluginspage="http://microsoft.com/windows/mediaplayer/en/download/"
					id="mediaPlayer" 
					name="mediaPlayer" 
					displaysize="4"
					autosize="-1" 
					bgcolor="darkblue" 
					showcontrols="true" 
					showtracker="-1" 
					showdisplay="0" 
					showstatusbar="-1" 
					videoborder3d="-1" 
					autostart="true"
					designtimesp="5311" 
					loop="true">
				</embed>
			</object>
		</td></tr>
		<tr><td style="text-align:center">
			'.AUTOGAL_LANG_L37.$fileSize.'
		</td></tr>
		<tr><td style="text-align:center">
			<a href="'.$src.'" target="_blank">'.AUTOGAL_LANG_L26.'</a>
		</td></tr>
		</table>';
	}
	else if ($eleInfo['isrealmedia'])
	{
		$pvWidth = ($eleInfo['viewwidth'] ? $eleInfo['viewwidth'] : AUTOGAL_MOVIEWIDTH);
		$pvHeight = ($eleInfo['viewheight'] ? $eleInfo['viewheight'] : AUTOGAL_MOVIEHEIGHT);
			
		$text = '
		<table border="0" cellpadding="0" align="center" width="'.$pvWidth.'" height="'.($pvHeight + 45).'">
        <tr><td>
		<object id="rvocx" 
			classid="clsid:CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA"
			width="'.$pvWidth.'" height="'.$pvHeight.'" 
			<param name="src" value="'.$eleInfo['url'].'">
			<param name="autostart" value="true">
			<param name="controls" value="imagewindow">
			<param name="console" value="video">
			<param name="loop" value="true">
			<embed src="'.$eleInfo['url'].'" 
				width="'.$pvWidth.'" height="'.$pvHeight.'" 
				loop="true" 
				type="audio/x-pn-realaudio-plugin" 
				controls="imagewindow" 
				console="video" 
				autostart="true">
			</embed>
		</object>
		<object id="rvocx" 
			classid="clsid:CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA"
			width="'.$pvWidth.'" height="30">
			<param name="src" value="'.$eleInfo['url'].'">
			<param name="autostart" value="true">
			<param name="controls" value="ControlPanel">
			<param name="console" value="video">
			<embed src="'.$eleInfo['url'].'" 
				width="'.$pvWidth.'" height="30" 
				controls="ControlPanel" 
				type="audio/x-pn-realaudio-plugin" 
				console="video" 
				autostart="true">
			</embed>
		</object>
		</td></tr>
		<tr><td style="text-align:center">
			'.AUTOGAL_LANG_L37.$fileSize.'
		</td></tr>
		<tr><td style="text-align:center">
			<a href="'.$eleInfo['url'].'" target="_blank">'.AUTOGAL_LANG_L26.'</a>
		</td></tr>
		</table><br />';
	}
	else
	{
		$imgUrl = $eleInfo['url'];
		$pvWidth = 0;
		$pvHeight = 0;
		$isLimited = 0;
		
		if (!$showFullImage)
		{
			$imageSize = AutoGal_GetImageDimensions($eleInfo['element']); 
			$imageWidth = $imageSize['w'];
			$imageHeight = $imageSize['h'];
		
			$whRatio = ($imageSize['w'] / $imageSize['h']);
			$hwRatio = ($imageSize['h'] / $imageSize['w']);
			
			# CHECK IMAGE DIMENSIONS TO SEE IF IT LARGER THAN MAX DISPLAY SIZE
			if (($imageWidth > AUTOGAL_MAXIMAGEWIDTH)||($imageHeight > AUTOGAL_MAXIMAGEHEIGHT))
			{
				$isLimited = 1;
				$targetWidth = AUTOGAL_MAXIMAGEWIDTH;
				$targetHeight = AUTOGAL_MAXIMAGEHEIGHT;
							
				if ($imageSize['w'] > $imageSize['h'])
				{
					$ratio = ($imageSize['w'] / $imageSize['h']);
					$targetHeight = round($targetWidth / $ratio);    
				}
				elseif ($imageSize['h'] > $imageSize['w']) 
				{
					$ratio = ($imageSize['h'] / $imageSize['w']);
					$targetWidth = round($targetHeight / $ratio); 
				} 
								
				# IF WE HAVE PREVIEW IMAGE RESIZING ON
				if (AUTOGAL_RESIZEPREVIEWIMGS)
				{
					$makePvImg = 0;
					# IF THE PREVIEW IMAGE EXISTS
					if (file_exists($eleInfo['imgpvpath']))
					{
						# CHECK TO SEE IF THE PREVIE IMAGE IS THE RIGHT SIZE
						$pvImageSize = AutoGal_GetImageDimensions($eleInfo['imgpvpath'], true);
						
						if (($pvImageSize['w'] != $targetWidth)||($pvImageSize['h'] != $targetHeight))
						{
							# NOT THE RIGHT SIZE? RESIZE IT.
							$makePvImg = 1;
						}
					}
					else
					{
						# PREVIEW IMAGE DOESN'T EXIT? MAKE ONE.
						$makePvImg = 1;
					}
					
					if ($makePvImg)
					{
						# RESIZE IMAGE TO THE PREVIEW IMAGE
						require_once(dirname(__FILE__)."/gdim_class.php");
						$gdim = new GDIM($mode, $imPath, $imQuality);
	
						$opts['keepaspect'] = 1;
						$opts['1stframe'] = 1;
						$opts['iflarger'] = 0;
						$opts['perms'] = AUTOGAL_PERMSGALTHUMBS;
						
						# RESIZE THE IMAGE
						if (!$gdim->resize($eleInfo['abspath'], $eleInfo['imgpvpath'], AUTOGAL_MAXIMAGEWIDTH, AUTOGAL_MAXIMAGEHEIGHT, $opts))
						{
							# SOMETHING FUCKED UP...
							global $ns;
							$ns->tablerender('', $gdim->lastError());
						}
						else
						{
							$imgUrl = $eleInfo['imgpvurl'];
						}
					}
					else
					{
						$imgUrl = $eleInfo['imgpvurl'];
					}
				}
				else
				{
					# NO PREVIEW IMAGE RESIZING? JUST RESIZE IT WITH HTML
					$pvWidth = $targetWidth;
					$pvHeight = $targetHeight;
				}
			}
		}
		
		# Create links for viewing full image
		$showFullAHref = "";
		$showFullAHrefEnd = "";
		if (($isLimited)&&($showLimitMsg))
		{
			if ($fullImgNewWin)
			{
				$showFullAHref = "<a target='_blank' href=\"".$eleInfo['url']."\">";
			}
			else
			{
				$showFullAHref = "<a href=\"".AUTOGAL_AUTOGALLERY."?show=".urlencode($eleInfo['element'])."&amp;full=1$shNewWin\">";
			}
				
			$showFullAHrefEnd = "</a>";
		}
		
		# Get the preview size from XML setting if it exists
		$pvWidth = ($eleInfo['viewwidth'] ? $eleInfo['viewwidth'] : $pvWidth);
		$pvHeight = ($eleInfo['viewheight'] ? $eleInfo['viewheight'] : $pvHeight);
		
		# Generate HTML style dimensions for image
		$imgDims = ";";
		if ($pvWidth) $imgDims .= "width:${pvWidth}px;";
		if ($pvHeight) $imgDims .= "height:${pvHeight}px;";
		$imgDims = substr($imgDims , 0, -1);

		# Generate embed link/code HTML
		if (AUTOGAL_SHOWEMBEDLINK)
		{
			$imageLink = "<img src=\"".AutoGal_ChangeToEmbed($eleInfo['url'])."\" alt=\"$title\" style=\"border:0$imgDims\" />";
			$inputEmbed = "<input class='tbox' name=\"ag_embedcode\" type=\"text\" value='".$imageLink."' readonly=\"true\" onfocus='this.select();this.focus();' onClick='this.select();this.focus();'>";
			$inputLink = "<input class='tbox' name=\"ag_embedlink\" type=\"text\" value='".AutoGal_ChangeToEmbed($eleInfo['url'])."' readonly=\"true\" onfocus='this.select();this.focus();' onClick='this.select();this.focus();'>";
			$embedHtmlCode = '<br /><table><tr><td><b>'.AUTOGAL_LANG_EMBED_L1.'</b>&#160;</td><td>'.$inputEmbed.'&#160;</td><tr><tr><td><b>'.AUTOGAL_LANG_EMBED_L2.'</b></td><td>'.$inputLink.'</td></tr></table>';
		}
		else
		{
			$embedHtmlCode = '';
		}
		
		# Generate preview HTML for image
		$text = 
		"$showFullAHref<img src=\"".htmlspecialchars($imgUrl)."\" alt=\"".htmlspecialchars($title)."\" style='border:0$imgDims' />$showFullAHrefEnd".
		"<br />"
		.($isLimited ? "$showFullAHref<span class='smalltext'>".str_replace("[SIZE]", $imageSize['w']."x".$imageSize['h'], AUTOGAL_LANG_L14)."</span></a><br />" : '').$embedHtmlCode;
	}
	
	return $text;
}

function AutoGal_ChangeToEmbed($text)
{
	$text_len = strlen($text);
	$fixed_text = "";
	for($i=0;$i<$text_len;$i++)
	{	
		if(substr($text,$i,1) == " ")
		{
			$fixed_text = $fixed_text . "%20";
		}
		else
		{
			$fixed_text = $fixed_text . substr($text,$i,1);
		}
	}
	return $fixed_text;
}

function AutoGal_NewWindowJS($type, $url, $title)
{
	$consts = get_defined_constants();
		
	$type = strtoupper($type);
	
	$width 			= $consts['AUTOGAL_'.$type.'NWINWIDTH'];
	$height 		= $consts['AUTOGAL_'.$type.'NWINHEIGHT'];
	$toolBar 		= ($consts['AUTOGAL_'.$type.'NWINTOOBAR'] ? "yes" : "no");
	$location 		= ($consts['AUTOGAL_'.$type.'NWINLOCBAR'] ? "yes" : "no");
	$directories 	= ($consts['AUTOGAL_'.$type.'NWINDIRECT'] ? "yes" : "no");
	$status 		= ($consts['AUTOGAL_'.$type.'NWINSTSBAR'] ? "yes" : "no");
	$menuBar 		= ($consts['AUTOGAL_'.$type.'NWINMNUBAR'] ? "yes" : "no");
	$scrollBars 	= ($consts['AUTOGAL_'.$type.'NWINSCRBAR'] ? "yes" : "no");
	$copyHistory 	= ($consts['AUTOGAL_'.$type.'NWINCPHIST'] ? "yes" : "no");
	$resizable 		= ($consts['AUTOGAL_'.$type.'NWINRESIZE'] ? "yes" : "no");
	$extraArgs 		= ($consts['AUTOGAL_'.$type.'NWINEXARGS'] ? "&".$consts['AUTOGAL_'.$type.'NWINEXARG'] : '');
		
	$windowArgs = "width=$width,height=$height,toolbar=$toolBar,location=$location,directories=$directories,status=$status,menubar=$menuBar,scrollbars=$scrollBars,copyhistory=$copyHistory,resizable=$resizable";
	
	# Farkin'
	$winOpenArgs = htmlspecialchars('"'.$url.'","'.$title.'","'.$windowArgs.'"', ENT_QUOTES);
	
	return 'window.open('.$winOpenArgs.')';
}

?>
