<?php
/*********************************************************************************************
 * e107 PLUGIN: Auto Gallery
 * VERSION:     2.xx
 * DESCRIPTION: A media/image gallery, where galleries are based on a directory structure. 
 *              Thumbnails are automatically generated through Imagemagick or GD.
 * WRITTEN BY:  Mr_Visible (www.cerebralsynergy.com)
 * DATE:        11/04/2007
 *
 * Released under the terms and conditions of the
 * GNU General Public License (http://gnu.org).
 ********************************************************************************************/
 
error_reporting(E_ALL);

if (!defined("AUTOGAL_CHARSET"))
{
	if (defined("CHARSET"))
	{
		define("AUTOGAL_CHARSET", CHARSET);
	}
	else
	{
		define("AUTOGAL_CHARSET", "utf-8");
	}
}

define(AUTOGAL_DIRCACHEDEBUG, 0);

class AutoGal_CGalleryList
{
	var $m_lastError;
	
	var $m_xmlParser;
	var $m_xmlTagTree;
	var $m_xmlCurrEle;
	var $m_xmlRawData;
	var $m_xmlReadData;
	var $m_xmlEleType;
	var $m_xmlFiles;
	var $m_xmlGalleries;
	
	var $m_galleries;
	var $m_files;
	var $m_cacheMethod;
	var $m_cacheFilename;
	var $m_gallery;
	var $m_galleryPath;
	var $m_sortOrder;
	var $m_doFileStat;
	var $m_recurse;
	
	var $m_cacheVars = array('element', 'ctime', 'mtime', 'size');
	
	#####################################################################
	# CONSTRUCTOR
	#####################################################################
	function AutoGal_CGalleryList($gallery, $opts=array())
	{
		$this->m_gallery = $gallery;
		$this->m_galleryPath = AutoGal_GetAbsGalPath($gallery);
		$this->m_cacheFilename = (isset($opts['cachefilename']) ?  $opts['cachefilename'] : '__cache.xml');
		$this->m_cacheMethod = (isset($opts['cachemethod']) ?  $opts['cachemethod'] : '');
		$this->m_sortOrder = (isset($opts['sortorder']) ?  $opts['sortorder'] : 'name');
		$this->m_recurse = (isset($opts['recurse']) ?  $opts['recurse'] : 0);
		
		if (($this->m_sortOrder == 'ctime')||($this->m_sortOrder == 'mtime')||($this->m_sortOrder == 'size'))
		{
			$this->m_doFileStat = true;
		}
		
		$this->m_doFileStat = (isset($opts['dofilestat']) ?  $opts['dofilestat'] : $this->m_doFileStat);
	}
	
	#####################################################################
	# OPTIONS
	#####################################################################
	function OptSortOrder($order)
	{
		if (isset($order)) $this->m_sortOrder = $order;
		return $this->m_sortOrder;
	}
	
	function OptRecurse($enable)
	{
		if (isset($enable)) $this->m_recurse = $enable;
		return $this->m_recurse;
	}
	
	function OptDoFileStat($enable)
	{
		if (isset($enable)) $this->m_doFileStat = $enable;
		return $this->m_doFileStat;
	}
	
	#####################################################################
	# SET/GET FUNCTIONS
	#####################################################################
	function Elements()
	{
		return array('galleries' => $this->Galleries(), 'files' => $this->Files());
	}
	
	function ElementsFlat()
	{
		return array_merge($this->Galleries(), $this->Files());
	}
	
	function ElementsMerged()
	{
		$eles = array_merge($this->Galleries(), $this->Files());
		$eles = AutoGal_SortEles($eles, $this->m_sortOrder);
		return $eles;
	}
	
	function Files()
	{
		return $this->m_files;
	}
	
	function Galleries()
	{
		return $this->m_galleries;
	}
	
	#####################################################################
	# ERROR HANDLING FUNCTIONS
	#####################################################################
	function LastError()
	{
		return $this->m_lastError;
	}
	
	function IsError()
	{
		return ($this->m_lastError ? 1 : 0);
	}
	
	function ClearError()
	{
		$this->m_lastError = '';
	}
	
	#####################################################################
	# DIR LISTING FUNCTIONS
	#####################################################################
	function ListGallery() 
	{
		$dir = $this->m_galleryPath;
		$recurse = $this->m_recurse;
		
		if ($recurse)
		{
			$eles = $this->DirListRecurseCached($dir);
		}
		else
		{
			$eles = $this->DirListCached($dir);
		}
		
		$this->m_files = $eles['files'];
		$this->m_galleries = $eles['galleries'];
		
		$this->m_files = AutoGal_SortEles($this->m_files, $this->m_sortOrder);
		$this->m_galleries = AutoGal_SortEles($this->m_galleries, $this->m_sortOrder);
	}
	
	function DirListCached($dir) 
	{
		if (AUTOGAL_DIRCACHEDEBUG) print "DIR LIST W/CACHE ($dir)...<br />";
		
		$eles = array();
		
		if ($this->m_cacheMethod == 'xml')
		{
			$cacheFile = $this->DirCacheFile($dir);
			
			if (!$this->DirCacheFileExists($dir))
			{
				if (AUTOGAL_DIRCACHEDEBUG) print "UPDATING CACHE (NO CACHE FILE EXISTS)...<br />";
				$eles = $this->DirListReal($dir);
				$this->WriteXMLCache($cacheFile, $eles);
			}
			else
			{
				$eles = $this->ReadXMLCache($cacheFile);
			}
		}
		else if ($this->m_cacheMethod == 'db')
		{
			if (AUTOGAL_DIRCACHEDEBUG) print "DATABASE CACHING<br />";
			# TODO - DATABASE CACHING
		}
		else
		{
			if (AUTOGAL_DIRCACHEDEBUG > 1) print "CACHE OFF<br />";
			$eles = $this->DirListReal($dir);
		}
		
		return $eles;
	}
	
	function DirListRecurseCached($dir) 
	{
		if (AUTOGAL_DIRCACHEDEBUG) print "LISTING GALLERY ($dir) RECURSIVELY...<br />";
		
		$galStack = array();
		$eles = array();
		
		$galStack[] = $dir;
		while ($galStack)
		{
			$currGallery = array_pop($galStack);
			$dirEles = $this->DirListCached($currGallery);
			
			$galleries = $dirEles['galleries'];
			foreach ($galleries as $eleInfo)
			{
				$galStack[] = $eleInfo['abspath'];
				if (AUTOGAL_DIRCACHEDEBUG > 1) print "ADD GAL TO STACK \"".$eleInfo['element']."\"<br />";
				$eles['galleries'][] = $eleInfo;
			}
			
			$files = $dirEles['files'];
			foreach ($files as $eleInfo)
			{
				$eles['files'][] = $eleInfo;
			}
		}
		
		return $eles;
	}
	
	function DirListReal($dir) 
	{
		if (AUTOGAL_DIRCACHEDEBUG) print "DIR LIST REAL ($dir)<br />";
		
		if (!$dh = opendir($dir))
		{
			$self->m_lastError = "Could not open directory '$dir' for reading!";
			return false;
		}
		
		$files = array();
		$galleries = array();

		while ($filename = readdir($dh))
		{
			if (($filename == '.')||($filename == '..')) continue;
			$filePath = $dir."/".$filename;
			
			if (is_file($filePath))
			{
				if (AutoGal_IsMediaFile($filePath))
				{	
					$element = AutoGal_GetElement($filePath);
					$eleInfo = AutoGal_GetElementInfo($element);
					
					if ($eleInfo)
					{
						if ($this->m_doFileStat)
						{
							if (AUTOGAL_DIRCACHEDEBUG > 1) print "GET FILE STAT \"".$eleInfo['element']."\"<br />";
							$fileInfo = stat($filePath);
							$eleInfo['ctime'] = $fileInfo['ctime'];
							$eleInfo['mtime'] = $fileInfo['mtime'];
							$eleInfo['size'] = $fileInfo['size'];
						}
						
						if (AUTOGAL_DIRCACHEDEBUG > 1) print "ADD FILE \"".$eleInfo['element']."\"<br />";
							
						$files[] = $eleInfo;
					}
				}
			}
			else
			{
				if (AutoGal_IsMediaDir($filePath))
				{
					$element = AutoGal_GetElement($filePath);
					$eleInfo = AutoGal_GetElementInfo($element);
					
					if ($eleInfo)
					{
						if ($this->m_doFileStat)
						{
							if (AUTOGAL_DIRCACHEDEBUG > 1) print "GET GALLERY STAT \"".$eleInfo['element']."\"<br />";
							$fileInfo = stat($filePath);
							$eleInfo['ctime'] = $fileInfo['ctime'];
							$eleInfo['mtime'] = $fileInfo['mtime'];
							$eleInfo['size'] = $fileInfo['size'];
						}
						
						if (AUTOGAL_DIRCACHEDEBUG > 1) print "ADD GALLERY \"".$eleInfo['element']."\"<br />";
						
						$galleries[] = $eleInfo;
					}
				}
			}
		}
		
		closedir($dh);
		return array('files' => $files, 'galleries' => $galleries);
	}
	
	#####################################################################
	# XML CACHING FUNCTIONS
	#####################################################################
	function DirCacheFile($dir)
	{
		return str_replace("//", "/", $dir."/".$this->m_cacheFilename);
	}
	
	function DirCacheFileExists($dir)
	{
		$file = $this->DirCacheFile($dir);
		return file_exists($file);
	}
		
	function ReadXMLCache($file)
	{
		$files = array();
		
		// CHECK IF XML PARSER EXISTS
		if(!function_exists('xml_parser_create'))
		{
			$this->m_lastError = AUTOGAL_LANG_METACLASS_L1;
			if (AUTOGAL_DIRCACHEDEBUG) print AUTOGAL_LANG_METACLASS_L2.$this->m_lastError."<br />\n";
			return false;
		}
		
		// READ FILE IN
		if (AUTOGAL_DIRCACHEDEBUG) print "READING CACHE FILE $file<br />";
		
		$HANDLE = fopen($file, "r");
		
		if (!$HANDLE)
		{
			$this->m_lastError = str_replace('[FILE]', $file, AUTOGAL_LANG_METACLASS_L3);
			if (AUTOGAL_DIRCACHEDEBUG) print AUTOGAL_LANG_METACLASS_L2.$this->m_lastError."<br />\n";
			return false;
		}
		
		$this->m_xmlRawData = fread($HANDLE, filesize($file));
		
		if (!$this->m_xmlRawData)
		{
			$this->m_lastError = str_replace('[FILE]', $file, AUTOGAL_LANG_METACLASS_L4);
			if (AUTOGAL_DIRCACHEDEBUG) print AUTOGAL_LANG_METACLASS_L2.$this->m_lastError."<br />\n";
			return false;
		}
		
		fclose($HANDLE);
		$this->ParseXML();
		
		if (AUTOGAL_DIRCACHEDEBUG) print "CACHE FILE READ<br />";
		
		return array('files' => $this->m_xmlFiles, 'galleries' => $this->m_xmlGalleries);
	}
	
	function ParseXML()
	{
		$this->m_xmlParser = xml_parser_create('');
		xml_set_object($this->m_xmlParser, $this);
		xml_set_element_handler($this->m_xmlParser, 'XMLStartTag', 'XMLEndTag');
		xml_set_character_data_handler($this->m_xmlParser, 'XMLData'); 
		xml_parse($this->m_xmlParser, $this->m_xmlRawData);
		
		xml_parser_free($this->m_xmlParser);
	
		return true;
	}
	
	function XMLStartTag($parser, $tag, $attrs)
	{
		$tag = strtolower($tag);
		$lastTag = $this->m_xmlTagTree[count($this->m_xmlTagTree) - 1];
		
		if (AUTOGAL_DIRCACHEDEBUG > 1) print "START LAST=$lastTag NOW=$tag<br>";
		
		if ($lastTag == 'dircache')
		{
			if ($tag == 'files')
			{
				if (AUTOGAL_DIRCACHEDEBUG > 1) print "READ IN FILES<br />";
				$this->m_xmlReadData = 'files';
			}
			else if ($tag == 'galleries')
			{
				if (AUTOGAL_DIRCACHEDEBUG > 1) print "READ IN FILES<br />";
				$this->m_xmlReadData = 'galleries';
			}
			elseif ($tag == 'lastupdate')
			{
				$this->m_xmlReadData = 'lastupdate';
			}
		}
		else if (($lastTag == 'files')||($lastTag == 'galleries'))
		{
			if ($tag == 'mediaobject')
			{
				if (AUTOGAL_DIRCACHEDEBUG > 1) print "READ MEDIA OBJECT ($lastTag)<br />";
				
				$this->m_xmlCurrEle = array();
				$this->m_xmlReadData = 'mediaobject';
				$this->m_xmlEleType = $lastTag;
			}
		}
		
		$this->m_xmlTagTree[] = $tag;
	}
		
	function XMLData($parser, $data)
	{
		$currTag = $this->m_xmlTagTree[count($this->m_xmlTagTree) - 1];
		
		$tagPath = implode("->", $this->m_xmlTagTree); 
		#if (AUTOGAL_DIRCACHEDEBUG > 1) print "DATA (".$this->m_xmlReadData.") [$tagPath]=\"$data\"<br />";
		
		if ($this->m_xmlReadData == 'mediaobject')
		{
			if (in_array($currTag, $this->m_cacheVars))
			{
				if (AUTOGAL_DIRCACHEDEBUG > 1) print "-->MEDIA OBJECT[$currTag] .= $data<br />";
				$this->m_xmlCurrEle[$currTag] .= $data;
			}
		}
		else if (($this->m_xmlReadData == 'lastupdate')&&($currTag == 'lastupdate'))
		{
			if (AUTOGAL_DIRCACHEDEBUG > 1) print "-->LAST UPDATE .= $data<br />";
			$this->m_cacheLastUpdate .= $data;
		}
		else
		{
			#if (AUTOGAL_DIRCACHEDEBUG > 1) print "NO READ<br />";
		}
	}
	
	function XMLEndTag($parser, $tag)
	{
		$tag = strtolower($tag);
		$lastTag = $this->m_xmlTagTree[count($this->m_xmlTagTree) - 1];
		
		#if (AUTOGAL_DIRCACHEDEBUG > 1) print "END $lastTag $tag<br>";
		
		if ($tag == 'mediaobject')
		{
			$element = $this->m_xmlCurrEle['element'];
			$eleInfo = AutoGal_GetElementInfo($element, false);
			
			if (AUTOGAL_DIRCACHEDEBUG > 1) print "END MEDIA OBJECT ($element)<br />";
			
			foreach ($this->m_cacheVars as $var)
			{
				if (($var != 'element')&&(isset($this->m_xmlCurrEle[$var])))
				{
					$eleInfo[$var] = $this->m_xmlCurrEle[$var];
				}
			}
				
			if ($this->m_xmlEleType == 'galleries')
			{
				$this->m_xmlGalleries[] = $eleInfo;
			}
			else
			{
				$this->m_xmlFiles[] = $eleInfo;
			}
			
			$this->m_xmlCurrEle = array();
			$this->m_xmlReadData = '';
			$this->m_xmlEleType = '';
		}
		else if ($tag == 'lastupdate')
		{
			$this->m_cacheLastUpdate = $this->DecodeDate($this->m_cacheLastUpdate);
			$this->m_xmlReadData = '';
		}
		
		array_pop($this->m_xmlTagTree);
	}
	
	function WriteXMLCache($file, $eles, $perms=666)
	{
		// BUILD XML META DATA
		$xmlData .=
		"<?xml version=\"1.0\" encoding=\"".AUTOGAL_CHARSET."\"?>\n\n".
		"<!-- Auto Gallery (www.cerebralsynergy.com) Directory Cache XML File -->\n\n".
		"<dircache>\n";
		
		$xmlData .=	"\t<lastupdate>".strftime("%H:%M:%S %d-%b-%Y", time())."</lastupdate>\n";
		
		$galleries = $eles['galleries'];
		$files = $eles['files'];
		
		if ($galleries)
		{
			$xmlData .=	"\t<galleries>\n";
			foreach ($galleries as $eleInfo)
			{
				$xmlData .= "\t\t<mediaobject>\n";
				
				foreach ($this->m_cacheVars as $attr)
				{
					$attrVal = $eleInfo[$attr];
					$xmlData .= "\t\t\t<$attr>".htmlspecialchars($attrVal)."</$attr>\n";
				}
				
				$xmlData .= "\t\t</mediaobject>\n";
			}
			$xmlData .= "\t</galleries>\n";
		}
		
		if ($files)
		{
			$xmlData .=	"\t<files>\n";
			foreach ($files as $eleInfo)
			{
				$xmlData .= "\t\t<mediaobject>\n";
				
				foreach ($this->m_cacheVars as $attr)
				{
					$attrVal = $eleInfo[$attr];
					$xmlData .= "\t\t\t<$attr>".htmlspecialchars($attrVal)."</$attr>\n";
				}
				
				$xmlData .= "\t\t</mediaobject>\n";
			}
			$xmlData .= "\t</files>\n";
		}
	
		$xmlData .= "</dircache>\n";
		
		$this->m_xmlRawData = $xmlData;
			
		// TOUCH AND CHMOD XML FILE IF NEW
		if (AUTOGAL_DIRCACHEDEBUG) print "WRITING $dir CACHE TO $file<br />";
		if (!file_exists($file))
		{
			touch($file);
			chmod($file, octdec($perms));
		}
		
		// WRITE XML TO FILE
		$HANDLE = fopen($file, "w+");
				
		if (!$HANDLE)
		{
			$this->m_lastError = str_replace('[FILE]', $file, AUTOGAL_LANG_METACLASS_L5);
			if (AUTOGAL_DIRCACHEDEBUG) print "ERROR: ".$this->m_lastError."<br />\n";
			flock($HANDLE, LOCK_UN);
			return false;
		}
		
		flock($HANDLE, LOCK_EX);
		if (fwrite($HANDLE, $xmlData) === false)
		{
			$this->m_lastError = str_replace('[FILE]', $file, AUTOGAL_LANG_METACLASS_L6);
			if (AUTOGAL_DIRCACHEDEBUG) print "ERROR: ".$this->m_lastError."<br />\n";
			flock($HANDLE, LOCK_UN);
			return false;
		}
		
		flock($HANDLE, LOCK_UN);
		fclose($HANDLE);
		
		if (!file_exists($file))
		{
			$this->m_lastError = str_replace('[FILE]', $file, AUTOGAL_LANG_METACLASS_L8);
			return false;
		}
		
		return true;
	}
	
	// WORK OUT EPOCH, CAN'T USE strptime 'COS IT'S PHP5 (NOONE SEEMS TO HAVE IT)
	function DecodeDate($dateStr)
	{
		if (preg_match("/^\s*(\d+)\:(\d+)\:(\d+)\s+(\d+)\-(\w+)\-(\d+)\s*$/", $dateStr, $dBits))
		{
			switch (strtolower($dBits[5]))
			{
				case 'jan': $monD = 1; break;
				case 'feb': $monD = 2; break;
				case 'mar': $monD = 3; break;
				case 'apr': $monD = 4; break;
				case 'may': $monD = 5; break;
				case 'jun': $monD = 6; break;
				case 'jul': $monD = 7; break;
				case 'aug': $monD = 8; break;
				case 'sep': $monD = 9; break;
				case 'oct': $monD = 10; break;
				case 'nov': $monD = 11; break;
				case 'dec': $monD = 12; break;
			}
			
			return mktime($dBits[1], $dBits[2], $dBits[3], $monD, $dBits[4], $dBits[6]);
		}
		else
		{
			return -1;
		}
	}
}

function AutoGal_SortEles($eles, $method='name')
{
	if (AUTOGAL_DIRCACHEDEBUG) print "SORTING ELEMENTS BY $method...<br />";
	
	if ($method == 'ctime')
	{
		usort($eles, "AutoGal_CmpElesByCTime");
	}
	else if ($method == 'mtime')
	{
		usort($eles, "AutoGal_CmpElesByMTime");
	}
	else if ($method == 'size')
	{
		usort($eles, "AutoGal_CmpElesBySize");
	}
	else
	{
		usort($eles, "AutoGal_CmpEles");
	}
	
	reset($eles);
	
	return $eles;
}

function AutoGal_CmpEles($a, $b)
{
   $cmpA = $a['sortby'];
   $cmpB = $b['sortby'];
      
   if ($cmpA == $cmpB) return 0;
   return ($cmpA < $cmpB) ? -1 : 1;
}

function AutoGal_CmpElesByCTime($a, $b)
{
   $cmpA = $a['ctime'];
   $cmpB = $b['ctime'];
      
   if ($cmpA == $cmpB) return 0;
   return ($cmpA > $cmpB) ? -1 : 1;
}

function AutoGal_CmpElesByMTime($a, $b)
{
   $cmpA = $a['mtime'];
   $cmpB = $b['mtime'];
      
   if ($cmpA == $cmpB) return 0;
   return (($cmpA > $cmpB) ? -1 : 1);
}

function AutoGal_CmpElesBySize($a, $b)
{
   $cmpA = $a['size'];
   $cmpB = $b['size'];
      
   if ($cmpA == $cmpB) return 0;
   return ($cmpA > $cmpB) ? -1 : 1;
}	

?>